var __assign = this && this.__assign || function () {
  __assign = Object.assign || function (t) {
    for (var s, i = 1, n = arguments.length; i < n; i++) {
      s = arguments[i];
      for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p)) t[p] = s[p];
    }
    return t;
  };
  return __assign.apply(this, arguments);
};
import React, { useEffect, useState } from 'react';
import { Divider, Dropdown, Menu, Space } from 'antd';
import MwButton, { addRefresh, hasPermission } from '../MwButton';
import MwAction from '../MwAction';
import { CTRL_DEFAULT_MAX, CTRL_DEFAULT_MORE_TEXT } from '../constant';
import { DownOutlined } from '@ant-design/icons';
import classNames from 'classnames';
import locale from '../locale';
import "./mw-ctrl.css";
/**
 * 返回一个控制项
 * @param node 节点
 * @param key key
 */
var getCtrlItem = function getCtrlItem(node, key, defaultProps) {
  var _a, _b;
  var props = __assign({}, node.props);
  if (((_a = node === null || node === void 0 ? void 0 : node.type) === null || _a === void 0 ? void 0 : _a.componentName) === 'MwAction' || ((_b = node === null || node === void 0 ? void 0 : node.type) === null || _b === void 0 ? void 0 : _b.componentName) === 'MwButton') {
    return /*#__PURE__*/React.createElement(MwAction, __assign({
      key: key,
      type: "link"
    }, defaultProps, props));
  }
  return node;
};
/**
 * 将子节点转化成 MwAction 按钮
 * @param children 子节点
 */
var getCtrlList = function getCtrlList(children, props) {
  var _a = props.max,
    max = _a === void 0 ? CTRL_DEFAULT_MAX : _a,
    sub = props.sub,
    _b = props.more,
    more = _b === void 0 ? /*#__PURE__*/React.createElement(React.Fragment, null, locale.table.actionRowMore || CTRL_DEFAULT_MORE_TEXT, " ", /*#__PURE__*/React.createElement(DownOutlined, null)) : _b;
  var ctrlList = [];
  if (!children.length || children.length === 1 && !children[0]) {
    return [];
  }
  // 如果节点只有一个元素
  if (children.length === 1) {
    return [getCtrlItem(children[0], 'key', {
      sub: sub
    })];
  }
  // 过滤掉无权限按钮和 null
  children = children.filter(function (node) {
    var _a;
    return node !== null && hasPermission((_a = node === null || node === void 0 ? void 0 : node.props) === null || _a === void 0 ? void 0 : _a.permission);
  });
  // 渲染没有折叠前的按钮
  for (var i = 0; i < (children.length <= max + 1 ? children.length : max); i++) {
    var node = children[i];
    if (!node) {
      continue;
    }
    var CtrlItem
    // 正常节点
    = void 0;
    // 正常节点
    CtrlItem = getCtrlItem(node, i, {
      sub: sub
    });
    // 添加这个节点
    ctrlList.push(CtrlItem);
  }
  // 多余的按钮会变成下拉菜单
  if (children.length > max + 1) {
    var menuList = [];
    for (var i = max; i < children.length; i++) {
      var node = children[i];
      if (!node) {
        continue;
      }
      var CtrlItem
      // 正常节点
      = void 0;
      // 正常节点
      CtrlItem = getCtrlItem(node, i, {
        __simple: true,
        type: 'text'
      });
      // 添加这个节点
      menuList.push( /*#__PURE__*/React.createElement(Menu.Item, {
        style: {
          padding: 0
        },
        key: max + i
      }, CtrlItem));
    }
    var menu = /*#__PURE__*/React.createElement(Menu, {
      style: {
        minWidth: 100
      }
    }, menuList);
    ctrlList.push( /*#__PURE__*/React.createElement(Dropdown, {
      key: max,
      overlay: menu,
      placement: "bottomRight"
    }, /*#__PURE__*/React.createElement(MwButton, {
      type: "link",
      className: classNames('mw-button', sub && 'sub')
    }, more)));
  }
  return ctrlList;
};
export default function MwCtrl(props) {
  var _a = useState(0),
    setRefresh = _a[1];
  var children = props.children,
    split = props.split,
    size = props.size,
    sub = props.sub,
    className = props.className,
    style = props.style;
  if (!Array.isArray(children)) {
    children = [children];
  }
  var ctrlList = getCtrlList(children, props);
  useEffect(function () {
    addRefresh(setRefresh);
  }, []);
  return /*#__PURE__*/React.createElement(Space, {
    size: size,
    className: classNames('mw-ctrl', className),
    style: style,
    split: split !== null && split !== void 0 ? split : /*#__PURE__*/React.createElement(Divider, {
      type: "vertical"
    })
  }, ctrlList);
}