var __assign = this && this.__assign || function () {
  __assign = Object.assign || function (t) {
    for (var s, i = 1, n = arguments.length; i < n; i++) {
      s = arguments[i];
      for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p)) t[p] = s[p];
    }
    return t;
  };
  return __assign.apply(this, arguments);
};
var __spreadArray = this && this.__spreadArray || function (to, from, pack) {
  if (pack || arguments.length === 2) for (var i = 0, l = from.length, ar; i < l; i++) {
    if (ar || !(i in from)) {
      if (!ar) ar = Array.prototype.slice.call(from, 0, i);
      ar[i] = from[i];
    }
  }
  return to.concat(ar || Array.prototype.slice.call(from));
};
import React, { useState, forwardRef, useImperativeHandle, useRef, useEffect, useMemo } from 'react';
import { Spin } from 'antd';
import MwDialog from '../MwDialog';
import MwForm from '../MwForm';
import { convertChildrenToField } from '../MwFields/convertFields';
import { omitObj } from '../utils';
import locale from '../locale';
/** 新增模式 */
export var MODE_ADD = 'add';
/** 修改模式 */
export var MODE_UPDATE = 'update';
/** 详情模式 */
export var MODE_VIEW = 'view';
/** 自定义模式 */
export var MODE_CUSTOM = 'custom';
/**
 * 过滤获得 form 的配置项
 * @param fields 配置项 (dialog-form)
 */
var getAyFormFields = function getAyFormFields(fields, mode, initParams, dialogOnly) {
  return fields.filter(function (field) {
    if (dialogOnly) {
      return !!field.dialog;
    } else {
      return true;
    }
  }).map(function (field) {
    var dialog = field.dialog;
    var formField = __assign(__assign(__assign({
      key: '_'
    }, omitObj(field, ['dialog'])), dialog), {
      _values: initParams
    });
    if (typeof formField.reSetting === 'function') {
      formField = formField.reSetting(formField, mode);
    }
    return formField;
  }).filter(function (field) {
    if (field && Array.isArray(field.hiddenMode) && mode) {
      return !field.hiddenMode.includes(mode);
    }
    return field;
  });
};
var getTitle = function getTitle(mode, title) {
  var _a;
  if (title) {
    return title;
  }
  var map = (_a = {}, _a[MODE_ADD] = locale.dialog.modeAdd, _a[MODE_UPDATE] = locale.dialog.modeUpdate, _a[MODE_VIEW] = locale.dialog.modeDetail, _a[MODE_CUSTOM] = locale.dialog.modeCustom, _a);
  return map[mode];
};
var dialogResolve;
var dialogReject;
/**
 * antd form 原生支持的方法尽数暴露出去
 */
var funcs = ['getFieldValue', 'getFieldsValue', 'getFieldError', 'getFieldsError', 'isFieldTouched', 'isFieldsTouched', 'isFieldValidating', 'resetFields', 'scrollToField', 'setFields', 'setFieldsValue', 'submit', 'validateFields'];
export default /*#__PURE__*/forwardRef(function MwDialogForm(props, ref) {
  var originFields = props.fields,
    title = props.title,
    addApi = props.addApi,
    updateApi = props.updateApi,
    span = props.span,
    width = props.width,
    name = props.name,
    beforeSubmit = props.beforeSubmit,
    dialogExtend = props.dialogExtend,
    formExtend = props.formExtend,
    drawer = props.drawer,
    dialogOnly = props.dialogOnly,
    _a = props.autoClose,
    autoClose = _a === void 0 ? true : _a,
    defaultVisible = props.visible,
    defaultMode = props.mode,
    _b = props.spinning,
    spinning = _b === void 0 ? false : _b,
    onClose = props.onClose,
    onCancel = props.onCancel,
    onSuccess = props.onSuccess,
    onError = props.onError,
    initialValues = props.initialValues,
    children = props.children;
  var fields = useMemo(function () {
    var childrenFields = convertChildrenToField(children);
    return __spreadArray(__spreadArray([], originFields || [], true), childrenFields, true);
  }, [originFields, children]);
  /** 弹窗是否可见 */
  var _c = useState(false),
    visible = _c[0],
    setVisible = _c[1];
  /** 当前所处于的模式 */
  var _d = useState(MODE_ADD),
    mode = _d[0],
    setMode = _d[1];
  /** 表单是否可以编辑 */
  var _e = useState(false),
    readonly = _e[0],
    setReadonly = _e[1];
  /** 是否正在保存中 */
  var _f = useState(false),
    loading = _f[0],
    setLoading = _f[1];
  /** 默认参数 */
  var _g = useState({}),
    initParams = _g[0],
    setInitParams = _g[1];
  /** form 需要的 fields */
  var _h = useState(getAyFormFields(fields, mode, initParams, dialogOnly)),
    formFields = _h[0],
    setFormFields = _h[1];
  /** form 控制 (需要主动调用里面的事件) */
  var formRef = useRef();
  /** 默认弹窗标题 */
  var _j = useState(title),
    dialogTitle = _j[0],
    setDialogTitle = _j[1];
  /** 打开弹窗的配置 */
  var _k = useState({}),
    config = _k[0],
    setConfig = _k[1];
  var debounce = function debounce(func, delay) {
    var timeout;
    return function () {
      var _this = this;
      var args = [];
      for (var _i = 0; _i < arguments.length; _i++) {
        args[_i] = arguments[_i];
      }
      clearTimeout(timeout);
      timeout = setTimeout(function () {
        func.apply(_this, args);
      }, delay);
    };
  };
  useEffect(function () {
    setFormFields(getAyFormFields(fields, mode, initParams, dialogOnly));
  }, [fields]);
  useEffect(function () {
    setDialogTitle(title);
  }, [title]);
  // 监听显示变化
  useEffect(function () {
    setVisible(!!defaultVisible);
    if (defaultVisible) {
      requestAnimationFrame(function () {
        var _a;
        (_a = formRef.current) === null || _a === void 0 ? void 0 : _a.resetFields();
      });
      if (initialValues) {
        requestAnimationFrame(function () {
          var _a;
          (_a = formRef.current) === null || _a === void 0 ? void 0 : _a.setFieldsValue(initialValues);
        });
      }
    }
  }, [defaultVisible, initialValues]);
  useEffect(function () {
    if (defaultMode) {
      setReadonly(defaultMode === MODE_VIEW);
      setMode(defaultMode);
    }
  }, [defaultMode]);
  /**
   * 初始化弹窗
   * @step 1、打开弹窗，清空只读
   * @step 2、如果有值，清空表单值
   * @step 3、如果有默认参数、设置默认参数
   * @param params 默认值
   * @param config 标题、fields 配置
   */
  var initDialog = function initDialog(params, config) {
    setReadonly(false);
    setConfig(config || {});
    if (config && config.fields) {
      formFields = getAyFormFields(config.fields, mode, initParams, dialogOnly);
      setFormFields(formFields);
      // 同步刷新 renderContent、render 函数
      useEffect(function () {
        setFormFields(getAyFormFields(config.fields, mode, initParams, dialogOnly));
      }, [config.fields]);
    } else {
      formFields = getAyFormFields(fields, mode, initParams, dialogOnly);
      setFormFields(formFields);
    }
    // 设置标题
    if (config && config.title) {
      setDialogTitle(config.title);
    }
    // 打开弹窗
    setVisible(true);
    // 第二次之后清空数据
    if (formRef.current) {
      formRef.current.resetFields();
    }
    // 设置默认值
    if (params) {
      setInitParams(params);
      requestAnimationFrame(function () {
        formRef.current.setFieldsValue(params);
      });
    } else {
      setInitParams({});
    }
  };
  var formInstans = {
    /**
     * 新增表单
     * @param params 默认值
     */
    add: function add(params, config) {
      return new Promise(function (resolve, reject) {
        dialogResolve = resolve;
        dialogReject = reject;
        mode = MODE_ADD;
        setMode(defaultMode || MODE_ADD);
        initDialog(params, config);
      });
    },
    /**
     * 修改表单
     * @param params 默认值
     */
    update: function update(params, config) {
      return new Promise(function (resolve, reject) {
        dialogResolve = resolve;
        dialogReject = reject;
        mode = MODE_UPDATE;
        setMode(defaultMode || MODE_UPDATE);
        initDialog(params, config);
      });
    },
    /**
     * 查看表单
     * @param params 默认值
     * @param config
     */
    view: function view(params, config) {
      mode = MODE_VIEW;
      setMode(defaultMode || MODE_VIEW);
      initDialog(params, config);
      setTimeout(function () {
        setReadonly(true);
      });
    },
    /**
     * 自定义表单，打开表单
     * @param params 默认值
     */
    open: function open(params, config) {
      mode = MODE_CUSTOM;
      setMode(defaultMode || MODE_CUSTOM);
      initDialog(params, config);
    },
    /**
     * 刷新
     */
    refreshFields: function refreshFields() {
      setFormFields(getAyFormFields(fields, mode, initParams, dialogOnly));
    },
    /**
     * 获取 form ref
     */
    getFormRef: function getFormRef() {
      return formRef;
    },
    /**
     * 关闭弹窗
     */
    closeDialog: function closeDialog() {
      _closeDialog();
    }
  };
  // 支持所有表单的方法
  funcs.forEach(function (func) {
    formInstans[func] = function () {
      var _a;
      var args = [];
      for (var _i = 0; _i < arguments.length; _i++) {
        args[_i] = arguments[_i];
      }
      return (_a = formRef.current)[func].apply(_a, args);
    };
  });
  // @ts-ignore 控制暴露出去的方法
  useImperativeHandle(ref, function () {
    return formInstans;
  });
  /**
   * 弹窗确定触发表单提交
   */
  var onConfirm = function onConfirm() {
    formRef.current.submit();
  };
  /**
   * 关闭弹窗
   */
  var _closeDialog = function _closeDialog() {
    setVisible(false);
    onClose && onClose();
    onCancel && onCancel();
  };
  /**
   * 表单提交
   * @step 1、根据不同模式获取不同的 API 接口
   * @step 2、开始 loading
   * @step 3、成功后 reolsve、关闭弹窗
   * @step 4、关闭 loading
   * @param values 提交参数
   */
  var handleSubmit = function handleSubmit(values) {
    var _a;
    if (loading) {
      return;
    }
    var apiMap = (_a = {}, _a[MODE_ADD] = addApi, _a[MODE_UPDATE] = updateApi, _a[MODE_CUSTOM] = config.api, _a);
    var api = apiMap[mode];
    if (api) {
      var params_1 = __assign(__assign({}, initParams), values);
      // 删除初始化的值携带
      for (var key in initParams) {
        if (key.startsWith('__')) {
          delete params_1[key];
        }
      }
      if (typeof beforeSubmit === 'function') {
        var result = beforeSubmit(params_1, mode);
        if (result !== false) {
          params_1 = result;
        } else {
          return;
        }
      }
      // 这个是兼容弹窗里面有编辑表格的时候，setLoading好像会触发重新渲染
      // 所以先这样处理
      !(props === null || props === void 0 ? void 0 : props.stopLoading) && setLoading(true);
      api(params_1).then(function (data) {
        if (dialogResolve) {
          dialogResolve({
            data: data,
            values: values,
            params: params_1,
            initParams: initParams,
            closeDialog: _closeDialog
          });
        }
        if (config.onSuccess) {
          config.onSuccess({
            data: data,
            values: values,
            params: params_1,
            initParams: initParams,
            closeDialog: _closeDialog
          });
        }
        if (onSuccess) {
          onSuccess({
            data: data,
            values: values,
            params: params_1,
            initParams: initParams,
            closeDialog: _closeDialog
          });
        }
        if (autoClose) {
          _closeDialog();
        }
      }, function (error) {
        if (dialogReject) {
          dialogReject({
            error: error,
            values: values,
            params: params_1,
            initParams: initParams,
            closeDialog: _closeDialog
          });
        }
        if (config.onError) {
          config.onError({
            error: error,
            values: values,
            params: params_1,
            initParams: initParams,
            closeDialog: _closeDialog
          });
        }
        if (onError) {
          onError({
            error: error,
            values: values,
            params: params_1,
            initParams: initParams,
            closeDialog: _closeDialog
          });
        }
      }).finally(function () {
        setLoading(false);
      });
    }
  };
  return /*#__PURE__*/React.createElement(MwDialog, __assign({
    width: width,
    title: getTitle(mode, dialogTitle),
    visible: visible,
    drawer: drawer,
    onConfirm: debounce(function () {
      !loading && onConfirm();
    }, 300),
    loading: loading,
    forceRender: true,
    onClose: _closeDialog,
    confirmLoading: loading,
    confirmVisible: mode !== MODE_VIEW && config.readonly !== true
  }, dialogExtend, config === null || config === void 0 ? void 0 : config.dialogExtend), /*#__PURE__*/React.createElement(Spin, {
    spinning: spinning
  }, /*#__PURE__*/React.createElement(MwForm, __assign({
    name: name,
    readonly: readonly,
    ref: formRef,
    fields: formFields,
    span: span || 22,
    onConfirm: handleSubmit
  }, formExtend))));
});