var __assign = this && this.__assign || function () {
  __assign = Object.assign || function (t) {
    for (var s, i = 1, n = arguments.length; i < n; i++) {
      s = arguments[i];
      for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p)) t[p] = s[p];
    }
    return t;
  };
  return __assign.apply(this, arguments);
};
var __rest = this && this.__rest || function (s, e) {
  var t = {};
  for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p) && e.indexOf(p) < 0) t[p] = s[p];
  if (s != null && typeof Object.getOwnPropertySymbols === "function") for (var i = 0, p = Object.getOwnPropertySymbols(s); i < p.length; i++) {
    if (e.indexOf(p[i]) < 0 && Object.prototype.propertyIsEnumerable.call(s, p[i])) t[p[i]] = s[p[i]];
  }
  return t;
};
var __spreadArray = this && this.__spreadArray || function (to, from, pack) {
  if (pack || arguments.length === 2) for (var i = 0, l = from.length, ar; i < l; i++) {
    if (ar || !(i in from)) {
      if (!ar) ar = Array.prototype.slice.call(from, 0, i);
      ar[i] = from[i];
    }
  }
  return to.concat(ar || Array.prototype.slice.call(from));
};
import React, { useEffect, useState } from 'react';
import { Button, Form, Space, Tooltip } from 'antd';
import locale from '../locale';
import { PlusOutlined, CopyOutlined, DeleteOutlined } from '@ant-design/icons';
import { FORM_TYPE_DATE, FORM_TYPE_DATE_RANGE, FORM_TYPE_LIST } from '../constant';
import { getDateValue } from './parseFields';
export default function MwFormList(props) {
  var field = props.field,
    getFormItem = props.getFormItem,
    formInstant = props.formInstant,
    ayFormProps = props.ayFormProps;
  var readonly = ayFormProps.readonly;
  // 最少行 & 最大行
  var _a = field.min,
    min = _a === void 0 ? 0 : _a,
    _b = field.max,
    max = _b === void 0 ? Infinity : _b;
  // 当前行数
  var _c = useState(0),
    recordNum = _c[0],
    setRecordNum = _c[1];
  useEffect(function () {
    setRecordNum(formInstant.getFieldValue(field.key).length);
  }, []);
  /**
   * 复制这行数据到末尾
   * @param name 实际上是当前行 index
   */
  var handleCopy = function handleCopy(name) {
    var _a;
    try {
      var value = formInstant.getFieldValue(field.key);
      var newValue = __spreadArray(__spreadArray([], value, true), [value[name]], false);
      setRecordNum(newValue.length);
      formInstant.setFieldsValue((_a = {}, _a[field.key || ''] = newValue, _a));
    } catch (_b) {
      console.error('复制失败');
    }
  };
  /**
   * 删除这行数据
   * @param name 实际上是当前行 index
   * @param remove 删除方法
   */
  var handleRemove = function handleRemove(name, remove) {
    remove(name);
    setRecordNum(Number(recordNum) - 1);
  };
  /**
   * 新增一行
   * @param add 新增方法
   */
  var handleAdd = function handleAdd(add) {
    add(field.creatorRecord || {});
    setRecordNum(Number(recordNum) + 1);
  };
  return /*#__PURE__*/React.createElement(Form.List, __assign({}, field.props, {
    name: field.key || field.label,
    key: field.key || field.label
  }), function (fields, _a) {
    var add = _a.add,
      remove = _a.remove;
    var value = readonly ? formInstant.getFieldValue(field.key) : [];
    return /*#__PURE__*/React.createElement(React.Fragment, null, fields.map(function (_a) {
      var key = _a.key,
        name = _a.name,
        restField = __rest(_a, ["key", "name"]);
      var children = field.children || [];
      if (!Array.isArray(children)) {
        children = [children];
      }
      var content = getFormItem(children.map(function (child) {
        var newField = __assign(__assign({}, child), {
          formItemProps: __assign(__assign(__assign({}, field.formItemProps), restField), {
            name: [name, child.key]
          })
        });
        if (readonly && !newField.render) {
          try {
            var text = value[name][child.key];
            // 日期或者日期区间
            if (newField.type === FORM_TYPE_DATE || Array.isArray(text) && newField.type === FORM_TYPE_DATE_RANGE) {
              text = getDateValue(text, newField, readonly);
            }
            newField.text = text;
          } catch (_a) {
            console.error('mw-form-list 转换错误');
          }
        }
        return newField;
      }), formInstant, ayFormProps, FORM_TYPE_LIST);
      return /*#__PURE__*/React.createElement(Space, __assign({
        key: "".concat(field.key, "-").concat(key),
        className: "mw-form-list-item",
        align: "end"
      }, field.spaceProps), content, !readonly && /*#__PURE__*/React.createElement(Space, {
        className: "mw-form-list-actions"
      }, recordNum < max && /*#__PURE__*/React.createElement("span", {
        className: "mw-form-list-action",
        onClick: function onClick() {
          return handleCopy(name);
        }
      }, /*#__PURE__*/React.createElement(Tooltip, {
        title: locale.form.copyToEnd
      }, /*#__PURE__*/React.createElement(CopyOutlined, null))), recordNum > min && /*#__PURE__*/React.createElement("span", {
        className: "mw-form-list-action",
        onClick: function onClick() {
          return handleRemove(name, remove);
        }
      }, /*#__PURE__*/React.createElement(Tooltip, {
        title: locale.form.removeRow
      }, /*#__PURE__*/React.createElement(DeleteOutlined, null)))));
    }), recordNum < max && !readonly && /*#__PURE__*/React.createElement(Button, {
      type: "dashed",
      onClick: function onClick() {
        return handleAdd(add);
      },
      icon: /*#__PURE__*/React.createElement(PlusOutlined, null)
    }, locale.form.addItem));
  });
}