var __assign = this && this.__assign || function () {
  __assign = Object.assign || function (t) {
    for (var s, i = 1, n = arguments.length; i < n; i++) {
      s = arguments[i];
      for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p)) t[p] = s[p];
    }
    return t;
  };
  return __assign.apply(this, arguments);
};
var __spreadArray = this && this.__spreadArray || function (to, from, pack) {
  if (pack || arguments.length === 2) for (var i = 0, l = from.length, ar; i < l; i++) {
    if (ar || !(i in from)) {
      if (!ar) ar = Array.prototype.slice.call(from, 0, i);
      ar[i] = from[i];
    }
  }
  return to.concat(ar || Array.prototype.slice.call(from));
};
import React, { useState, useEffect, forwardRef, useImperativeHandle, useRef } from 'react';
import { List, Space, Card } from 'antd';
import { TABLE_DEFAULT_ROW_KEY, TABLE_PAGESIZE, TABLE_START_PAGE } from '../constant';
import { clearEmpty, getKey } from '../utils';
import { defaultDataFilter, defaultSearchFilter } from '../MwTable';
import { MwListContext } from './context';
import locale from '../locale';
export default /*#__PURE__*/forwardRef(function MwList(props, ref) {
  var className = props.className,
    header = props.header,
    api = props.api,
    data = props.data,
    children = props.children,
    title = props.title,
    listHeader = props.listHeader,
    renderItem = props.renderItem,
    listExtend = props.listExtend,
    onLoad = props.onLoad,
    _a = props.rowKey,
    rowKey = _a === void 0 ? TABLE_DEFAULT_ROW_KEY : _a,
    autoload = props.autoload,
    filterData = props.filterData,
    beforeSearch = props.beforeSearch,
    pagination = props.pagination,
    defaultSearchValue = props.defaultSearchValue,
    extendSearchParams = props.extendSearchParams,
    btnBefore = props.btnBefore,
    onParamsChange = props.onParamsChange;
  /** 是否已经初始化 */
  var initRef = useRef(false);
  /** 表格查询的数据 */
  var _b = useState({
      pagination: {
        pageSize: (pagination === null || pagination === void 0 ? void 0 : pagination.pageSize) || TABLE_PAGESIZE,
        current: (pagination === null || pagination === void 0 ? void 0 : pagination.current) || TABLE_START_PAGE
      },
      filters: {},
      sorts: [],
      search: clearEmpty(defaultSearchValue || {})
    }),
    loadParams = _b[0],
    setLoadParams = _b[1];
  /** 表格数据 */
  var _c = useState(data || []),
    tableData = _c[0],
    _setTableData = _c[1];
  /** 是否正在加载 */
  var _d = useState(false),
    loading = _d[0],
    setLoading = _d[1];
  /** 总共多少条 */
  var _e = useState(0),
    total = _e[0],
    setTotal = _e[1];
  /** 禁用的选项的 key */
  var _f = useState([]),
    disabledKeys = _f[0],
    setDisabledKeys = _f[1];
  useEffect(function () {
    _setTableData(data || []);
  }, [data]);
  useEffect(function () {
    if (onParamsChange) {
      onParamsChange(loadParams);
    }
    if (initRef.current) {
      // 如果已经初始化，变动触发加载数据
      loadData();
    } else if (autoload !== false && !initRef.current) {
      // 尚未初始化 且 默认加载，触发加载数据
      loadData();
    }
  }, [loadParams]);
  useEffect(function () {
    initRef.current = true;
  }, []);
  /**
   * 获得查询前的参数
   */
  var getApiParams = function getApiParams() {
    var searchParams = __assign(__assign({}, loadParams), {
      search: __assign(__assign({}, extendSearchParams), loadParams.search)
    });
    // 默认筛选过滤
    if (defaultSearchFilter) {
      searchParams = defaultSearchFilter(searchParams);
    }
    // 主动筛选过滤
    if (beforeSearch) {
      searchParams = beforeSearch(searchParams);
    }
    return searchParams;
  };
  /**
   * 加载数据
   * @step 1、获得 params
   * @step 2、开始 loading
   * @step 3、加载数据
   * @step 4、设置表格数据
   * @step 5、关闭 loading
   */
  var loadData = function loadData() {
    if (api) {
      var searchParams = getApiParams();
      setLoading(true);
      api(searchParams).then(function (data) {
        data = defaultDataFilter(data);
        var content = data.content;
        if (filterData) {
          content = filterData(data);
        }
        _setTableData(content);
        setTotal(data.totalCount || content.length);
        if (onLoad) {
          onLoad(content, data);
        }
      }).finally(function () {
        setLoading(false);
      });
    }
  };
  /**
   * 改变表格查询数据
   * @param pageSize 每页多少条
   * @param current 当前第几页
   * @param search 查询数据
   */
  var updateLoadParams = function updateLoadParams(_a) {
    var pageSize = _a.pageSize,
      current = _a.current,
      search = _a.search;
    var newLoadParams = __assign({}, loadParams);
    if (pageSize !== undefined) {
      newLoadParams.pagination.pageSize = pageSize;
    }
    if (current !== undefined) {
      newLoadParams.pagination.current = current;
    }
    if (search !== undefined) {
      newLoadParams.search = clearEmpty(search);
    }
    setLoadParams(newLoadParams);
  };
  var handleTableChange = function handleTableChange(page, size) {
    var newParams = __assign({}, loadParams);
    newParams.pagination = {
      pageSize: size,
      current: page
    };
    setLoadParams(newParams);
  };
  useImperativeHandle(ref, function () {
    return {
      /**
       * 刷新页面
       */
      refresh: function refresh() {
        loadData();
      },
      /**
       * 回到第一页，刷新页面
       */
      reset: function reset(search) {
        updateLoadParams({
          search: search,
          current: TABLE_START_PAGE
        });
      },
      /**
       * 获取表格数据
       */
      getTableData: function getTableData() {
        return tableData;
      },
      /**
       * 设置表格数据
       */
      setTableData: function setTableData(tableData) {
        _setTableData(tableData);
      },
      /**
       * 根据 id 删除某一行数据
       */
      deleteRowByKey: function deleteRowByKey(key) {
        // @ts-ignore
        var newTableData = __spreadArray([], tableData, true);
        var index = newTableData.findIndex(function (row) {
          return getKey(row, rowKey) === key;
        });
        if (index >= 0) {
          newTableData.splice(index, 1);
          _setTableData(newTableData);
        }
      },
      getApiParams: getApiParams
    };
  });
  useEffect(function () {
    _setTableData(data || []);
  }, [data]);
  /**
   * 是否拥有头部
   */
  var hasHeader = function hasHeader() {
    if (title || btnBefore) {
      return true;
    }
    // 检查是否拥有除了 MwFields 以外的子元素
    if (children) {
      var checkChildren = Array.isArray(children) ? __spreadArray([], children, true) : [children];
      // @ts-ignore
      if (checkChildren.filter(function (node) {
        var _a;
        return ((_a = node === null || node === void 0 ? void 0 : node.type) === null || _a === void 0 ? void 0 : _a.componentName) !== 'MwFields';
      }).length > 0) {
        return true;
      }
    }
    return false;
  };
  return /*#__PURE__*/React.createElement(MwListContext.Provider, {
    value: {
      data: tableData,
      disabledKeys: disabledKeys,
      setDisabledKeys: setDisabledKeys
    }
  }, /*#__PURE__*/React.createElement(Card, {
    className: "mw-list ".concat(className || '')
  }, hasHeader() ? /*#__PURE__*/React.createElement("header", {
    className: "mw-list-header"
  }, /*#__PURE__*/React.createElement("div", {
    className: "mw-list-header-left"
  }, /*#__PURE__*/React.createElement(Space, null, typeof title === 'string' ? /*#__PURE__*/React.createElement("h2", {
    className: "mw-list-title"
  }, title) : title)), /*#__PURE__*/React.createElement("div", {
    className: "mw-list-header-right"
  }, /*#__PURE__*/React.createElement(Space, null, btnBefore, children))) : null, header, listHeader, /*#__PURE__*/React.createElement(List, __assign({
    itemLayout: "horizontal",
    dataSource: tableData,
    loading: loading,
    pagination: pagination !== false ? __assign(__assign({
      showTotal: function showTotal(total) {
        return "".concat(locale.table.totalBefore, " ").concat(total, " ").concat(locale.table.totalAfter);
      },
      showQuickJumper: true
    }, pagination), {
      total: total,
      onChange: handleTableChange,
      current: loadParams.pagination.current,
      pageSize: loadParams.pagination.pageSize
    }) : false,
    renderItem: renderItem
  }, listExtend))));
});