var __assign = this && this.__assign || function () {
  __assign = Object.assign || function (t) {
    for (var s, i = 1, n = arguments.length; i < n; i++) {
      s = arguments[i];
      for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p)) t[p] = s[p];
    }
    return t;
  };
  return __assign.apply(this, arguments);
};
var __rest = this && this.__rest || function (s, e) {
  var t = {};
  for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p) && e.indexOf(p) < 0) t[p] = s[p];
  if (s != null && typeof Object.getOwnPropertySymbols === "function") for (var i = 0, p = Object.getOwnPropertySymbols(s); i < p.length; i++) {
    if (e.indexOf(p[i]) < 0 && Object.prototype.propertyIsEnumerable.call(s, p[i])) t[p[i]] = s[p[i]];
  }
  return t;
};
import React, { useState, useRef, forwardRef, useImperativeHandle, useMemo, useLayoutEffect } from 'react';
import MwForm from '../MwForm';
import MwButton from '../MwButton';
import { Form, Col, Space, Card } from 'antd';
import { SearchOutlined, ReloadOutlined, DownOutlined, UpOutlined } from '@ant-design/icons';
import classNames from 'classnames';
import { FORM_TYPE_DATE, FORM_TYPE_DATE_RANGE } from '../constant';
import locale from '../locale';
/** 不同尺寸下，每个配置项所占用的 span */
var SizeMap = {
  large: 6,
  middle: 8,
  small: 12,
  mini: 24
};
/** 默认展示 n 行 */
var defaultVisibleRow = 2;
/**
 * 设置 mini 状态下默认展示的行数
 * @param row 展示行数
 */
export var setSearchDefaultVisibleRow = function setSearchDefaultVisibleRow(row) {
  if (row > 0) {
    defaultVisibleRow = row;
  }
};
/**
 * 获取 field 当前的位置，默认位置是 1，越往前越靠前
 * @param field form 配置项
 */
var getOrder = function getOrder(field, i) {
  return field.order === undefined ? i : field.order;
};
/**
 * 将查询的 field 转成 form 的field
 * @param fields 查询配置项
 * @param mini 是否缩小
 */
var getSearchFields = function getSearchFields( /** 配置项 */
fields, /** 是否收起 */
mini, /** 当前尺寸 */
size, /** 每个项所占用的 span */
calcSpan, /** 默认展开 n 行 */
visibleRow, /** 是否平铺展示 */
inline) {
  // 累计的 span
  var spanSum = 0;
  // 每一行占用的 span，超过 24 清 0
  var rowSpanSum = 0;
  var newFields = fields
  // 过滤掉隐藏的
  .filter(function (field) {
    return field.hidden !== true;
  })
  // 获取到排序值
  .map(function (field, i) {
    return __assign(__assign({}, field), {
      order: getOrder(field, i)
    });
  });
  // 排序
  newFields.sort(function (a, b) {
    return a.order - b.order;
  });
  // 生成 MwForm 的 field
  newFields = newFields.map(function (field, index) {
    // @ts-ignore
    var newField = __assign({}, field);
    // 平铺展示
    if (inline) {
      // 去掉
      newField.title = '';
      if (![FORM_TYPE_DATE, FORM_TYPE_DATE_RANGE].includes(field.type || '')) {
        // 添加 props
        newField.props = __assign({
          // @ts-ignore
          placeholder: field.title || ''
        }, field.props);
      }
      return newField;
    }
    // 当前条默认的 span 值
    var newSpan = field.grid ? field.grid[size] : calcSpan;
    // 如果这一行超过 24 格子，意味着要换行
    if (newSpan + rowSpanSum > 24) {
      // 填补折行的空白空间
      spanSum += 24 - rowSpanSum;
      // 重新调整 rowSpanSum 的值，保证值 < 24
      rowSpanSum = 24 - newSpan;
    } else {
      // 累计 这一行的 span 值
      rowSpanSum += newSpan;
    }
    // 如果刚好 = 24，意味则换行，清 0
    if (rowSpanSum === 24) {
      rowSpanSum = 0;
    }
    spanSum += newSpan;
    if (mini) {
      // 如果超过多行，直接隐藏
      if (spanSum > visibleRow * 24 - calcSpan && index !== 0) {
        newField.hidden = true;
      }
    }
    // MwForm 的 span 值是具体数值
    newField.span = newSpan;
    return newField;
  });
  return {
    searchFields: newFields,
    span: spanSum
  };
};
var getMiniLabel = function getMiniLabel(mini) {
  return mini ? locale.search.collapsed : locale.search.expand;
};
/**
 * ant form 原生支持的方法尽数暴露出去
 */
var funcs = ['getFieldValue', 'getFieldsValue', 'getFieldError', 'getFieldsError', 'isFieldTouched', 'isFieldsTouched', 'isFieldValidating', 'resetFields', 'scrollToField', 'setFields', 'setFieldsValue', 'submit', 'validateFields'];
export default /*#__PURE__*/forwardRef(function MwSearch(props, ref) {
  var fields = props.fields,
    onConfirm = props.onConfirm,
    onReset = props.onReset,
    formExtend = props.formExtend,
    defaultOpen = props.defaultOpen,
    toggleVisible = props.toggleVisible,
    _a = props.visibleRow,
    visibleRow = _a === void 0 ? defaultVisibleRow : _a,
    actionVisible = props.actionVisible,
    inline = props.inline,
    otherProps = __rest(props, ["fields", "onConfirm", "onReset", "formExtend", "defaultOpen", "toggleVisible", "visibleRow", "actionVisible", "inline"]);
  var wrapRef = useRef();
  var timerRef = useRef(0);
  // 是否是 mini 状态
  var _b = useState(!defaultOpen),
    mini = _b[0],
    setMini = _b[1];
  // 当前尺寸
  var _c = useState('large'),
    size = _c[0],
    setSize = _c[1];
  // 计算的格子大小
  var calcSpan = useMemo(function () {
    return SizeMap[size];
  }, [size]);
  // 显示的 fields
  var _d = getSearchFields(fields, mini, size, calcSpan, visibleRow, inline || false),
    searchFields = _d.searchFields,
    visibleSpan = _d.span;
  // 查询按钮所占的 span 值
  var actionSpan = useMemo(function () {
    if (inline) {
      return undefined;
    }
    if (visibleSpan <= 18) {
      return 6;
    }
    // 累计的 span
    var spanSum = 0;
    // 每一行占用的 span，超过 24 清 0
    var rowSpanSum = 0;
    // mini 状态下的 span 值
    var miniSpanSum = 0;
    searchFields.forEach(function (field, index) {
      var newSpan = field.span || calcSpan;
      // 如果这一行超过 24 格子，意味着会这行
      if (newSpan + rowSpanSum > 24) {
        // 填补折行的空白空间
        spanSum += 24 - rowSpanSum;
        // 重新调整 rowSpanSum 的值，保证值 < 24
        rowSpanSum = 24 - newSpan;
      } else {
        // 累计 这一行的 span 值
        rowSpanSum += newSpan;
      }
      // 如果刚好 = 24，意味则换行，清 0
      if (rowSpanSum === 24) {
        rowSpanSum = 0;
      }
      // 如果超过指定行数，直接隐藏
      if (spanSum + newSpan + calcSpan > visibleRow * 24 && !miniSpanSum && index !== 0) {
        miniSpanSum = spanSum;
      }
      spanSum += newSpan;
    });
    if (miniSpanSum === 0) {
      miniSpanSum = spanSum;
    }
    var span = 24 - (mini ? miniSpanSum : spanSum) % 24;
    return span;
  }, [searchFields, calcSpan, mini, visibleSpan, inline]);
  // 是否应该右侧
  var actionRight = useMemo(function () {
    return visibleSpan >= 24 - calcSpan;
  }, [visibleSpan]);
  // 查询区域样式
  var actionStyle = useMemo(function () {
    var style = {};
    if (actionRight) {
      style.textAlign = 'right';
    } else {
      style.paddingLeft = 50;
    }
    if (inline) {
      style = {};
    }
    return style;
  }, [actionRight, inline]);
  /** 控制 any form 的实例 */
  var formRef = useRef();
  /** 暴露出去的 form 的实例，允许父组件通过 ref 调用方法 */
  var formInstans = {
    resize: function resize() {
      return handleResize();
    }
  };
  /** 填充方法 */
  funcs.forEach(function (func) {
    formInstans[func] = function () {
      var _a;
      var args = [];
      for (var _i = 0; _i < arguments.length; _i++) {
        args[_i] = arguments[_i];
      }
      return (_a = formRef.current)[func].apply(_a, args);
    };
  });
  useImperativeHandle(ref, function () {
    return formInstans;
  });
  /**
   * 切换展开
   */
  var toggleMini = function toggleMini() {
    setMini(!mini);
  };
  /**
   * 获取每个 field 的 span
   */
  var getSpanByWrap = function getSpanByWrap() {
    try {
      var width = wrapRef.current.offsetWidth;
      var size_1 = 'large';
      if (width > 1300) {
        size_1 = 'large';
      } else if (width > 900) {
        size_1 = 'middle';
      } else if (width > 700) {
        size_1 = 'small';
      } else {
        size_1 = 'mini';
      }
      setSize(size_1);
    } catch (_a) {}
  };
  /**
   * 监听大小变化事件
   */
  var handleResize = function handleResize() {
    // 节流，每 100 毫秒执行一次
    var now = Date.now();
    if (now - timerRef.current > 100) {
      getSpanByWrap();
      timerRef.current = now;
    }
  };
  useLayoutEffect(function () {
    // @ts-ignore 监听元素宽度变化
    var observe = new window.ResizeObserver(function () {
      handleResize();
    });
    observe.observe(wrapRef.current);
    return function () {
      observe.disconnect();
    };
  }, []);
  /**
   * 重置
   */
  var handleReset = function handleReset() {
    formRef.current.resetFields();
    formRef.current.submit();
    onReset && onReset();
  };
  /**
   * 提交查询
   * @param values 返回参数
   */
  var handleConfirm = function handleConfirm(values) {
    if (onConfirm) {
      onConfirm(values);
    }
  };
  /**
   * 展开缩小切换
   */
  var ToogleBtn = function ToogleBtn() {
    return /*#__PURE__*/React.createElement(MwButton, {
      type: "link",
      onClick: toggleMini
    }, getMiniLabel(mini), mini ? /*#__PURE__*/React.createElement(DownOutlined, null) : /*#__PURE__*/React.createElement(UpOutlined, null));
  };
  return /*#__PURE__*/React.createElement(Card, {
    className: classNames('mw-search', inline ? '' : 'full-width', inline ? 'inline' : '')
  }, /*#__PURE__*/React.createElement("div", {
    className: "mw-search-content",
    ref: wrapRef
  }, /*#__PURE__*/React.createElement(MwForm, __assign({
    layout: {
      labelCol: {
        flex: '100px'
      }
    },
    ref: formRef,
    fields: searchFields,
    span: calcSpan,
    gutter: inline ? 0 : 16,
    formLayout: inline ? 'inline' : 'horizontal',
    onConfirm: handleConfirm
  }, otherProps, formExtend), actionVisible !== false && /*#__PURE__*/React.createElement(Col, {
    span: actionSpan,
    style: actionStyle
  }, /*#__PURE__*/React.createElement(Form.Item, null, /*#__PURE__*/React.createElement(Space, null, /*#__PURE__*/React.createElement(MwButton, {
    htmlType: "submit",
    type: "primary",
    icon: /*#__PURE__*/React.createElement(SearchOutlined, null)
  }, locale.search.search), /*#__PURE__*/React.createElement(MwButton, {
    icon: /*#__PURE__*/React.createElement(ReloadOutlined, null),
    onClick: handleReset
  }, locale.search.reset), toggleVisible !== false && !inline ? visibleSpan > visibleRow * 24 - (actionSpan || 0) && searchFields.length > 1 && /*#__PURE__*/React.createElement(ToogleBtn, null) : null))))));
});