var __assign = this && this.__assign || function () {
  __assign = Object.assign || function (t) {
    for (var s, i = 1, n = arguments.length; i < n; i++) {
      s = arguments[i];
      for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p)) t[p] = s[p];
    }
    return t;
  };
  return __assign.apply(this, arguments);
};
var __spreadArray = this && this.__spreadArray || function (to, from, pack) {
  if (pack || arguments.length === 2) for (var i = 0, l = from.length, ar; i < l; i++) {
    if (ar || !(i in from)) {
      if (!ar) ar = Array.prototype.slice.call(from, 0, i);
      ar[i] = from[i];
    }
  }
  return to.concat(ar || Array.prototype.slice.call(from));
};
import React, { useRef, forwardRef, useImperativeHandle, useState, useMemo } from 'react';
import MwSearch from '../MwSearch';
import MwForm from '../MwForm';
import MwList from '../MwList';
import MwDialogForm from '../MwDialogForm';
import useSelection from '../MwSearchTable/use/useSelection';
import { isObj } from '../utils';
import { getDefaultValue } from '../MwForm';
import { Space } from 'antd';
import { getActionProps } from '../MwAction';
import MwButton from '../MwButton';
import { MwSearchTableContext } from '../MwSearchTable/context';
import useExtraBtn from '../MwSearchTable/use/useExtraBtn';
import { convertChildrenToField } from '../MwFields/convertFields';
import Selection from './Selection';
import SelectionAll from './SelectionAll';
import "./mw-search-list.css";
export { Selection, SelectionAll };
/**
 * 转化并过滤成 mw-search 能用的 fields
 * @param fields 查询表格的 fields
 */
var getSearchFields = function getSearchFields(fields) {
  var searchFields = [];
  var moreSearchFields = [];
  fields.filter(function (field) {
    return isObj(field.search) || field.search === true;
  }).forEach(function (field) {
    var search = typeof field.search === 'boolean' ? {} : field.search;
    if (!search) {
      return {
        title: '配置有误',
        key: 'xxx',
        type: 'input'
      };
    }
    var searchField = __assign({
      title: field.title,
      key: search.key || field.key || '',
      type: field.type || 'input',
      options: field.options || []
    }, search);
    if (field.children) {
      searchField.children = field.children;
    }
    if (searchField.position === 'more') {
      moreSearchFields.push(searchField);
    } else {
      searchFields.push(searchField);
    }
  });
  return {
    searchFields: searchFields,
    moreSearchFields: moreSearchFields
  };
};
/**
 * 判断该节点是否只出现在底部
 * @param node MwAction 按钮
 */
var isFooterActionOnly = function isFooterActionOnly(node) {
  if (!node || !node.props) {
    return false;
  }
  var props = getActionProps(node.props, {});
  return props.tableFooterExtraOnly === true;
};
/**
 * 获取表格底部以及右侧 MwAction 按钮
 * @param node MwAction 按钮
 */
var getTableActionBtns = function getTableActionBtns(children) {
  /** 右侧按钮 */
  var footerActions = [];
  /** 底部按钮 */
  var rightActions = [];
  if (Array.isArray(children)) {
    children.forEach(function (node) {
      if (isFooterActionOnly(node)) {
        footerActions.push(node);
      } else if (node) {
        rightActions.push(node);
      }
    });
  } else {
    if (isFooterActionOnly(children)) {
      footerActions.push(children);
    } else if (children) {
      rightActions.push(children);
    }
  }
  return {
    rightActions: rightActions,
    footerActions: footerActions
  };
};
function MwSearchList(props, ref) {
  var originFields = props.fields,
    api = props.api,
    deleteApi = props.deleteApi,
    children = props.children,
    data = props.data,
    title = props.title,
    ctrl = props.ctrl,
    selectionType = props.selectionType,
    onSelectionChange = props.onSelectionChange,
    rowKey = props.rowKey,
    dialogFormExtend = props.dialogFormExtend,
    scrollX = props.scrollX,
    height = props.height,
    filterData = props.filterData,
    beforeSearch = props.beforeSearch,
    selectShowKey = props.selectShowKey,
    center = props.center,
    onLoad = props.onLoad,
    searchVisible = props.searchVisible,
    listExtend = props.listExtend,
    pagination = props.pagination,
    btnBefore = props.btnBefore,
    extendSearchParams = props.extendSearchParams,
    after = props.after,
    before = props.before,
    editMode = props.editMode,
    autoload = props.autoload,
    rowSelection = props.rowSelection,
    searchExtend = props.searchExtend,
    renderItem = props.renderItem,
    onParamsChange = props.onParamsChange,
    listHeader = props.listHeader;
  var fields = useMemo(function () {
    var childrenFields = convertChildrenToField(children);
    return __spreadArray(__spreadArray([], originFields || [], true), childrenFields, true);
  }, [originFields, children]);
  /** form 控制 */
  var formRef = useRef();
  /** table 控制 */
  var tableRef = useRef();
  /** search 控制 */
  var searchRef = useRef();
  /** search 控制 */
  var moreSearchRef = useRef();
  /** 查询项 */
  var _a = getSearchFields(fields),
    searchFields = _a.searchFields,
    moreSearchFields = _a.moreSearchFields;
  /** 列表项 */
  var _b = useState([]),
    tableFields = _b[0],
    setTableFields = _b[1];
  /** 使用勾选 */
  var _c = useSelection({
      rowKey: rowKey,
      selectionType: selectionType,
      onSelectionChange: onSelectionChange,
      selectShowKey: selectShowKey,
      rowSelection: rowSelection
    }),
    header = _c.header,
    message = _c.message,
    tableRowSelection = _c.tableRowSelection,
    selection = _c.selection,
    _clearSelection = _c.clearSelection,
    setSelection = _c.setSelection,
    addSelection = _c.addSelection,
    removeSelection = _c.removeSelection;
  /** action 展示，底部 or 右侧 */
  var _d = getTableActionBtns(children),
    footerActions = _d.footerActions,
    rightActions = _d.rightActions;
  var _e = useExtraBtn(tableRef, searchRef, tableFields, setTableFields, __assign(__assign({}, props), {
      extraSizeVisible: false,
      extraSettingVisible: false
    })),
    extraBtns = _e.extraBtns,
    size = _e.size,
    isEnter = _e.isEnter;
  /** 查询完成，刷新列表 */
  var onConfirm = function onConfirm() {
    var _a, _b;
    // 更多查询数据
    var moreSearchValues = ((_a = moreSearchRef.current) === null || _a === void 0 ? void 0 : _a.getFieldsValue()) || {};
    // 头顶查询数据
    var searchValues = ((_b = searchRef.current) === null || _b === void 0 ? void 0 : _b.getFieldsValue()) || {};
    // 合并查询
    tableRef.current.reset(__assign(__assign({}, moreSearchValues), searchValues));
  };
  /** 暴露方法 */
  useImperativeHandle(ref, function () {
    return {
      /**
       * 刷新页面
       */
      refresh: function refresh() {
        tableRef.current.refresh();
      },
      /**
       * 回到第一页，刷新页面
       */
      reset: function reset() {
        // 跟点击【查询】逻辑一致
        onConfirm();
      },
      /**
       * 清空选项
       */
      clearSelection: function clearSelection() {
        _clearSelection();
      },
      /**
       * 获取 search 对象
       */
      getSearchRef: function getSearchRef() {
        return searchRef.current;
      },
      /**
       * 获取 moreSearch 对象，表格右侧查询框
       */
      getMoreSearchRef: function getMoreSearchRef() {
        return moreSearchRef.current;
      },
      /**
       * 获取已经选中的对象
       */
      getSelection: function getSelection() {
        return selection;
      },
      /**
       * 设置选中的行
       */
      setSelection: setSelection,
      /**
       * 添加选中的行
       */
      addSelection: addSelection,
      /**
       * 获取表格数据
       */
      getTableData: function getTableData() {
        return tableRef.current.getTableData();
      },
      /**
       * 清空过滤
       * @param keys
       */
      clearFilters: function clearFilters(keys) {
        if (keys === void 0) {
          keys = [];
        }
        return tableRef.current.clearFilters(keys);
      },
      /**
       * 设置过滤值
       * @param filters 需要设置的过滤值，{ key: value } 格式
       */
      setFiltersValue: function setFiltersValue(filters) {
        return tableRef.current.setFiltersValue(filters);
      },
      /**
       * 设置排序值
       * @param sorts [{ key, order }] 组成格式
       */
      setSortsValue: function setSortsValue(sorts) {
        return tableRef.current.setSortsValue(sorts);
      },
      /**
       * 清空排序
       * @param keys
       */
      clearSorts: function clearSorts(keys) {
        if (keys === void 0) {
          keys = [];
        }
        return tableRef.current.clearSorts(keys);
      },
      /** 获取请求前参数 */
      getApiParams: function getApiParams() {
        return tableRef.current.getApiParams();
      }
    };
  });
  var listProps = {
    ref: tableRef,
    rowSelection: tableRowSelection,
    listHeader: listHeader,
    api: api,
    size: size,
    data: data,
    title: title,
    ctrl: ctrl,
    rowKey: rowKey,
    scrollX: scrollX,
    height: height,
    filterData: filterData,
    beforeSearch: beforeSearch,
    onLoad: onLoad,
    listExtend: listExtend,
    pagination: pagination,
    // @ts-ignore
    defaultSearchValue: getDefaultValue(__spreadArray(__spreadArray([], searchFields, true), moreSearchFields, true)),
    btnBefore: btnBefore,
    editMode: editMode,
    extendSearchParams: extendSearchParams,
    autoload: autoload,
    onParamsChange: onParamsChange
  };
  /** 列表子元素 */
  var tableChildren = useMemo(function () {
    var children = [];
    if (moreSearchFields && moreSearchFields.length) {
      children.push( /*#__PURE__*/React.createElement(MwForm, {
        className: "mw-search-list-more",
        key: "mw-search-list-more",
        ref: moreSearchRef,
        fields: moreSearchFields,
        onConfirm: onConfirm
      }, /*#__PURE__*/React.createElement(MwButton, {
        className: "mw-search-list-more-submit",
        htmlType: "submit"
      })));
    }
    if (rightActions && rightActions.length) {
      children.push(rightActions);
    }
    if (extraBtns) {
      children.push(extraBtns);
    }
    if (children.length === 1) {
      return children[0];
    }
    return children.length ? children : null;
  }, [moreSearchRef, moreSearchFields, onConfirm, rightActions, extraBtns]);
  return /*#__PURE__*/React.createElement("div", {
    className: "mw-search-list ".concat(isEnter ? 'full' : '')
  }, /*#__PURE__*/React.createElement(MwSearchTableContext.Provider, {
    value: {
      formRef: formRef,
      tableRef: tableRef,
      selection: selection,
      deleteApi: deleteApi,
      rowKey: rowKey,
      setSelection: setSelection,
      clearSelection: _clearSelection,
      addSelection: addSelection,
      removeSelection: removeSelection,
      searchTableRef: ref
    }
  }, before, searchVisible !== false && searchFields.length > 0 ? /*#__PURE__*/React.createElement(MwSearch, __assign({
    ref: searchRef,
    fields: searchFields,
    onConfirm: onConfirm
  }, searchExtend)) : null, center, dialogFormExtend ? /*#__PURE__*/React.createElement(MwDialogForm, __assign({
    ref: formRef,
    dialogOnly: true
  }, dialogFormExtend), children) : null, /*#__PURE__*/React.createElement(MwList, __assign({}, listProps, {
    fields: tableFields,
    renderItem: renderItem,
    header: header
  }), tableChildren), selection.length && footerActions.length ? /*#__PURE__*/React.createElement("div", {
    className: "mw-search-list-footer-actions"
  }, message, /*#__PURE__*/React.createElement(Space, null, footerActions)) : null, after));
}
var component = /*#__PURE__*/forwardRef(MwSearchList);
// @ts-ignore
component.Selection = Selection;
// @ts-ignore
component.SelectionAll = SelectionAll;
export default component;