var __assign = this && this.__assign || function () {
  __assign = Object.assign || function (t) {
    for (var s, i = 1, n = arguments.length; i < n; i++) {
      s = arguments[i];
      for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p)) t[p] = s[p];
    }
    return t;
  };
  return __assign.apply(this, arguments);
};
var __spreadArray = this && this.__spreadArray || function (to, from, pack) {
  if (pack || arguments.length === 2) for (var i = 0, l = from.length, ar; i < l; i++) {
    if (ar || !(i in from)) {
      if (!ar) ar = Array.prototype.slice.call(from, 0, i);
      ar[i] = from[i];
    }
  }
  return to.concat(ar || Array.prototype.slice.call(from));
};
import React, { useState, useEffect } from 'react';
import { Tag, Popover, Alert } from 'antd';
import MwAction from '../../MwAction';
import { getKey } from '../../utils';
import locale from '../../locale';
export default function useSelection(_props) {
  var rowKey = _props.rowKey,
    selectionType = _props.selectionType,
    onSelectionChange = _props.onSelectionChange,
    selectShowKey = _props.selectShowKey,
    rowSelection = _props.rowSelection;
  var _a = useState([]),
    selectionKeys = _a[0],
    setSelectionKeys = _a[1];
  var _b = useState([]),
    selection = _b[0],
    setSelection = _b[1];
  var tableRowSelection;
  if (selectionType) {
    tableRowSelection = __assign(__assign({}, rowSelection), {
      type: selectionType,
      selectedRowKeys: selectionKeys,
      onSelect: function onSelect(record, selected) {
        if (selectionType === 'radio') {
          changeRadioSelection(record);
        } else {
          selected ? addSelection(record) : removeSelection(null, record);
        }
      },
      onSelectAll: function onSelectAll(selected, selectedRows, changeRows) {
        selected ? addSelectionArray(selectedRows) : removeSelectionArray(changeRows);
      }
    });
  }
  /**
   * 清空所有选项
   */
  var clearSelection = function clearSelection() {
    setSelectionKeys([]);
    setSelection([]);
  };
  /**
   * 设置选中的行
   */
  var setDefaultSelection = function setDefaultSelection(selection) {
    setSelection(selection);
    setSelectionKeys(selection.map(function (row) {
      return getKey(row, rowKey);
    }));
  };
  /**
   * 添加选项
   */
  var addDefaultSelection = function addDefaultSelection(addSelection) {
    // @ts-ignore
    var newSelection = __spreadArray([], selection, true);
    addSelection.forEach(function (row) {
      if (!selectionKeys.includes(getKey(row, rowKey))) {
        newSelection.push(row);
      }
    });
    setSelection(newSelection);
    setSelectionKeys(newSelection.map(function (row) {
      return getKey(row, rowKey);
    }));
  };
  var changeRadioSelection = function changeRadioSelection(row) {
    var newKeys = [];
    var newSelection = [];
    newKeys.push(getKey(row, rowKey));
    newSelection.push(row);
    setSelectionKeys(newKeys);
    setSelection(newSelection);
  };
  /**
   * 添加选项（单个）
   * @param row 某一条选项
   */
  var addSelection = function addSelection(row) {
    // @ts-ignore
    var newKeys = __spreadArray([], selectionKeys, true);
    var newSelection = __spreadArray([], selection, true);
    newKeys.push(getKey(row, rowKey));
    newSelection.push(row);
    setSelectionKeys(newKeys);
    setSelection(newSelection);
  };
  /**
   * 添加选项（数组）
   * @param rows 项目列表
   */
  var addSelectionArray = function addSelectionArray(rows) {
    var newKeys = __spreadArray([], selectionKeys, true);
    var newSelection = __spreadArray([], selection, true);
    rows.forEach(function (row) {
      if (!row) {
        return;
      }
      var key = getKey(row, rowKey);
      if (!newKeys.includes(key)) {
        newKeys.push(key);
        newSelection.push(row);
      }
    });
    setSelectionKeys(newKeys);
    setSelection(newSelection);
  };
  /**
   * 移除某个选项
   * @param i 移除选项的 index
   */
  var removeSelection = function removeSelection(i, record) {
    var newKeys = __spreadArray([], selectionKeys, true);
    var newSelection = __spreadArray([], selection, true);
    if (i === null && record) {
      i = newKeys.findIndex(function (key) {
        return key === getKey(record, rowKey);
      });
    }
    if (typeof i === 'number') {
      newKeys.splice(i, 1);
      newSelection.splice(i, 1);
    }
    setSelectionKeys(newKeys);
    setSelection(newSelection);
  };
  /**
   * 移除一组选项
   * @param rows 移除选项
   */
  var removeSelectionArray = function removeSelectionArray(rows) {
    var newKeys = __spreadArray([], selectionKeys, true);
    var newSelection = __spreadArray([], selection, true);
    rows.forEach(function (row) {
      var index = newKeys.findIndex(function (item) {
        return item === getKey(row, rowKey);
      });
      if (index >= 0) {
        newKeys.splice(index, 1);
        newSelection.splice(index, 1);
      }
    });
    setSelectionKeys(newKeys);
    setSelection(newSelection);
  };
  /** Popover 弹窗的提示 */
  var popContent = /*#__PURE__*/React.createElement("div", {
    className: "mw-search-poper"
  }, selection.map(function (row, i) {
    return /*#__PURE__*/React.createElement(Tag, {
      key: getKey(row, rowKey),
      closable: true,
      className: "mb",
      onClose: function onClose() {
        return removeSelection(i);
      }
    }, row[selectShowKey || 'name']);
  }));
  var message = /*#__PURE__*/React.createElement("div", null, /*#__PURE__*/React.createElement("span", null, locale.table.selectedBefore, /*#__PURE__*/React.createElement(Popover, {
    title: locale.table.selectedTitle,
    content: popContent
  }, /*#__PURE__*/React.createElement("a", null, selection.length)), "\xA0", locale.table.selectedAfter), /*#__PURE__*/React.createElement(MwAction, {
    className: "ml",
    type: "link",
    size: "small",
    onClick: clearSelection
  }, locale.table.selectedClear));
  /** 头部已选中的提示 */
  var header = selectionKeys.length ? /*#__PURE__*/React.createElement(Alert, {
    className: "mw-search-table-alert",
    message: message,
    showIcon: true
  }) : '';
  useEffect(function () {
    if (onSelectionChange) {
      onSelectionChange(selection, selectionKeys);
    }
  }, [onSelectionChange, selection]);
  return {
    header: header,
    message: message,
    tableRowSelection: tableRowSelection,
    selection: selection,
    clearSelection: clearSelection,
    removeSelection: removeSelection,
    setSelection: setDefaultSelection,
    addSelection: addDefaultSelection
  };
}