var __assign = this && this.__assign || function () {
  __assign = Object.assign || function (t) {
    for (var s, i = 1, n = arguments.length; i < n; i++) {
      s = arguments[i];
      for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p)) t[p] = s[p];
    }
    return t;
  };
  return __assign.apply(this, arguments);
};
import React, { useRef, useEffect } from 'react';
import moment from 'moment';
import { Tooltip, Input, Image, Tag } from 'antd';
import { FORM_READONLY_EMPTY } from '../constant';
import { getValueByOptions, renderStatus } from '../utils';
import MwSelect from '../MwSelect';
export var install = function install(registerTableRender) {
  registerTableRender('__options', function (_a) {
    var field = _a.field,
      text = _a.text;
    var option = field.options.find(function (option) {
      return option.value === text;
    });
    return option ? option.label : text;
  });
  registerTableRender('__ellipsis', function (_a) {
    var text = _a.text,
      field = _a.field;
    var type = field.type,
      _b = field.options,
      options = _b === void 0 ? [] : _b;
    var shiftTitle = text;
    if (type === 'select' && options.length) {
      shiftTitle = getValueByOptions(text, field.options);
    }
    if (type === 'number' && typeof text === 'number' && !isNaN(text)) {
      shiftTitle = text.toString();
    }
    return /*#__PURE__*/React.createElement(Tooltip, {
      placement: field.placement || 'topLeft',
      title: shiftTitle
    }, /*#__PURE__*/React.createElement("span", null, shiftTitle || ''));
  });
  registerTableRender('datetime', function (_a) {
    var text = _a.text,
      field = _a.field;
    if (!text) {
      return '';
    }
    return moment(text).format(field.format || 'YYYY-MM-DD HH:mm:ss');
  });
  registerTableRender('editable-cell-input', function (_a) {
    var text = _a.text,
      field = _a.field;
    return function (_a) {
      var editing = _a.editing,
        mode = _a.mode,
        save = _a.save;
      var inputRef = useRef(null);
      useEffect(function () {
        if (editing && mode === 'col') {
          inputRef.current.focus();
        }
      }, [editing]);
      return !editing ? text : /*#__PURE__*/React.createElement(Input, __assign({
        placeholder: "\u8BF7\u8F93\u5165"
      }, field.contentProps, {
        ref: inputRef,
        onBlur: save,
        onPressEnter: save
      }));
    };
  });
  registerTableRender('editable-cell-select', function (_a) {
    var text = _a.text,
      field = _a.field;
    var options = field.options || [];
    var label = '';
    if (Array.isArray(text)) {
      if (!text.length) {
        text = FORM_READONLY_EMPTY;
      }
      label = text.map(function (item) {
        return getValueByOptions(item, field.options);
      }).join(field.splitText || '、');
    } else {
      label = getValueByOptions(text, field.options);
    }
    return function (_a) {
      var editing = _a.editing,
        save = _a.save,
        mode = _a.mode;
      var selectRef = useRef(null);
      useEffect(function () {
        if (editing && mode === 'col') {
          selectRef.current.focus();
        }
      }, [editing]);
      return !editing ? label : /*#__PURE__*/React.createElement(MwSelect, __assign({
        placeholder: "\u8BF7\u9009\u62E9",
        style: {
          width: '100%'
        }
      }, field.contentProps, {
        ref: selectRef,
        options: options,
        onBlur: save
      }));
    };
  });
  registerTableRender('image', function (_a) {
    var text = _a.text,
      field = _a.field;
    return /*#__PURE__*/React.createElement(Image, __assign({
      width: 100
    }, field.props, {
      src: text
    }));
  });
  registerTableRender('html', function (_a) {
    var text = _a.text,
      field = _a.field;
    return /*#__PURE__*/React.createElement("div", {
      dangerouslySetInnerHTML: {
        __html: text
      }
    });
  });
  registerTableRender('tags', function (_a) {
    var text = _a.text,
      field = _a.field;
    if (!Array.isArray(text) || !field.colorMap) {
      return text;
    }
    return text.map(function (item) {
      return /*#__PURE__*/React.createElement(Tag, {
        key: item,
        color: field.colorMap[item]
      }, item);
    });
  });
  registerTableRender('unit', function (_a) {
    var text = _a.text,
      field = _a.field;
    return /*#__PURE__*/React.createElement("div", null, field.prefix, text, field.suffix);
  });
  /**
   * 5
   * @decs 状态加文字
   *
   * @returns ReactNode
   */
  registerTableRender('status', function (_a) {
    var text = _a.text,
      field = _a.field;
    var _b = field.options,
      options = _b === void 0 ? [] : _b;
    return renderStatus(text, options, field.type);
  });
};
export default {
  install: install
};