var __assign = this && this.__assign || function () {
  __assign = Object.assign || function (t) {
    for (var s, i = 1, n = arguments.length; i < n; i++) {
      s = arguments[i];
      for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p)) t[p] = s[p];
    }
    return t;
  };
  return __assign.apply(this, arguments);
};
import React from 'react';
import { TABLE_CTRL_KEY } from '../constant';
import { Tooltip } from 'antd';
import { QuestionCircleOutlined } from '@ant-design/icons';
import "./mw-table.css";
export var install = function install(renderMap) {
  /**
   * 转化获得 field
   * @param field table Field
   * @param params 查询参数
   * @param tableData 表格数据
   * @param setTableData 设置表格数据
   * @param props 表格属性
   */
  var getAyTableField = function getAyTableField(field, params, tableData, setTableData, props) {
    // 支持 tooltip 属性
    var title = field.__alias || field.title;
    if (field.tooltip) {
      title = /*#__PURE__*/React.createElement("span", null, title, /*#__PURE__*/React.createElement(Tooltip, {
        placement: "top",
        title: field.tooltip
      }, /*#__PURE__*/React.createElement(QuestionCircleOutlined, {
        style: {
          marginLeft: 4
        }
      })));
    }
    var tableField = __assign(__assign({
      key: field.key,
      dataIndex: field.key
    }, field), {
      title: title
    });
    if (field.render) {
      tableField.render = field.render;
    }
    if (Array.isArray(field.children)) {
      field.children = field.children.map(function (field) {
        return getAyTableField(field, params, tableData, setTableData);
      });
    }
    // options 自动注册
    if (field.options && !field.render && !tableField.renderType) {
      tableField.renderType = '__options';
    }
    // 处理筛选
    if (field.filter && field.options) {
      tableField.filters = field.filters || JSON.parse(JSON.stringify(field.options).replace(/"label"/g, '"text"'));
      tableField.filteredValue = params.filters[field.key];
      field.filterMultiple = field.filterMultiple || false;
    }
    // 处理排序
    if (field.sort) {
      var sorts = params.sorts;
      // 寻找在 params 存在的排序
      var sortItem = sorts.find(function (item) {
        return item.key === field.key;
      });
      if (!sortItem) {
        // 不存在直接清空
        tableField.sortOrder = false;
      } else {
        // 存在排序，则值为设置后的排序
        tableField.sortOrder = sortItem.order;
      }
      tableField.sorter = field.sorter || {
        multiple: field.sortOrder
      };
    }
    // 多余显示 ...
    if (field.ellipsis) {
      tableField.ellipsis = {
        showTitle: false
      };
      tableField.renderType = '__ellipsis';
    }
    // 处理可编辑行
    if (field.editable) {
      tableField.renderType = field.renderType ? field.renderType : 'editable-cell-input';
      tableField.onCell = function (record) {
        // 多行编辑下，优先执行配置上的属性
        var fieldCellProps = {};
        if (field.onCell) {
          fieldCellProps = field === null || field === void 0 ? void 0 : field.onCell(record);
        }
        return __assign(__assign({}, fieldCellProps), {
          record: record,
          field: field,
          tableData: tableData,
          setTableData: setTableData,
          tableProps: props
        });
      };
    }
    if (!tableField.render && renderMap[tableField.renderType] && typeof renderMap[tableField.renderType] === 'function') {
      tableField.render = function (text, record, index) {
        return renderMap[tableField.renderType]({
          text: text,
          record: record,
          index: index,
          field: tableField
        });
      };
    }
    return tableField;
  };
  /**
   * 重新过滤配置项
   * @param fields 配置项目
   * @param params 查询参数
   * @param tableData 表格数据
   * @param setTableData 设置表格数据
   * @param ctrl 操作列
   * @param props 表格属性
   */
  var getAyTableFields = function getAyTableFields(fields, params, tableData, setTableData, ctrl, props) {
    var tableFields = fields.filter(function (field) {
      if (field.__extraTouched) {
        return field.__hidden === false;
      }
      if (typeof field.hidden === 'function') {
        return field.hidden();
      }
      return field.hidden !== true;
    }).map(function (field) {
      return getAyTableField(field, params, tableData, setTableData, props);
    });
    // 保证操作列在最后
    if (ctrl && ctrl.render && tableFields.every(function (field) {
      return field.key !== TABLE_CTRL_KEY;
    })) {
      ctrl.key = TABLE_CTRL_KEY;
      ctrl.title = ctrl.title || '操作';
      ctrl.order = 999;
      ctrl.__order = 999;
      tableFields.push(ctrl);
    }
    // 排序
    tableFields = tableFields.sort(function (a, b) {
      return a.order - b.order;
    });
    // 二次排序
    if (tableFields.some(function (field) {
      return field.__extraTouched;
    })) {
      tableFields = tableFields.sort(function (a, b) {
        return (a.__order || 0) - ((b === null || b === void 0 ? void 0 : b.__order) || 0);
      });
    }
    return tableFields;
  };
  return {
    getAyTableField: getAyTableField,
    getAyTableFields: getAyTableFields
  };
};
export default {
  install: install
};