var __assign = this && this.__assign || function () {
  __assign = Object.assign || function (t) {
    for (var s, i = 1, n = arguments.length; i < n; i++) {
      s = arguments[i];
      for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p)) t[p] = s[p];
    }
    return t;
  };
  return __assign.apply(this, arguments);
};
var __spreadArray = this && this.__spreadArray || function (to, from, pack) {
  if (pack || arguments.length === 2) for (var i = 0, l = from.length, ar; i < l; i++) {
    if (ar || !(i in from)) {
      if (!ar) ar = Array.prototype.slice.call(from, 0, i);
      ar[i] = from[i];
    }
  }
  return to.concat(ar || Array.prototype.slice.call(from));
};
import React, { useState, useEffect, forwardRef, useImperativeHandle, useRef, useMemo } from 'react';
import { Table, Space, Card, Pagination } from 'antd';
import core from './core';
import RenderMapInit from './RenderMapInit';
import { getComponents } from './EditableTable';
import { clearEmpty } from '../utils';
import { TABLE_DEFAULT_ROW_KEY, TABLE_PAGESIZE, TABLE_START_PAGE } from '../constant';
import { getKey } from '../utils';
import { MwListContext } from '../MwList/context';
import locale from '../locale';
import { useAntdResizableHeader } from '@minko-fe/use-antd-resizable-header';
import '@minko-fe/use-antd-resizable-header/dist/index.css';
import "./mw-table.css";
/** 默认请求前列表过滤 */
export var defaultSearchFilter = function defaultSearchFilter(params) {
  return params;
};
/** 默认请求后列表过滤 */
export var defaultDataFilter = function defaultDataFilter(params) {
  return params;
};
/** 自定义请求前过滤 */
export var setDefaultSearchFilter = function setDefaultSearchFilter(cb) {
  defaultSearchFilter = cb;
};
/** 自定义请求后过滤 */
export var setDefaultDataFilter = function setDefaultDataFilter(cb) {
  defaultDataFilter = cb;
};
/** 自定义渲染模板 */
var renderMap = {};
/** 注册自定义渲染模板 */
export var registerTableRender = function registerTableRender(key, render) {
  renderMap[key] = render;
};
/** 安装默认渲染模板 */
RenderMapInit.install(registerTableRender);
/** 获取表格渲染列 */
export var getAyTableFields = core.install(renderMap).getAyTableFields;
/** 表格默认属性 */
var tableDefaultProps = {};
/** 设置表格默认属性 */
export var setTableDefaultProps = function setTableDefaultProps(props) {
  tableDefaultProps = props;
};
export default /*#__PURE__*/forwardRef(function MwTable(props, ref) {
  var className = props.className,
    rowClassName = props.rowClassName,
    fields = props.fields,
    header = props.header,
    api = props.api,
    size = props.size,
    data = props.data,
    children = props.children,
    title = props.title,
    rowSelection = props.rowSelection,
    ctrl = props.ctrl,
    onLoad = props.onLoad,
    _a = props.rowKey,
    rowKey = _a === void 0 ? TABLE_DEFAULT_ROW_KEY : _a,
    scrollX = props.scrollX,
    autoload = props.autoload,
    filterData = props.filterData,
    beforeSearch = props.beforeSearch,
    onExpand = props.onExpand,
    pagination = props.pagination,
    tableExtend = props.tableExtend,
    defaultSearchValue = props.defaultSearchValue,
    defaultSortsValue = props.defaultSortsValue,
    defaultFiltersValue = props.defaultFiltersValue,
    extendSearchParams = props.extendSearchParams,
    btnBefore = props.btnBefore,
    height = props.height,
    editMode = props.editMode,
    tableHeader = props.tableHeader,
    onParamsChange = props.onParamsChange,
    getSearchParams = props.getSearchParams,
    useOriginPagination = props.useOriginPagination;
  /** 是否已经初始化 */
  var initRef = useRef(false);
  /** 表格查询的数据 */
  var _b = useState({
      pagination: {
        pageSize: (pagination === null || pagination === void 0 ? void 0 : pagination.pageSize) || TABLE_PAGESIZE,
        current: (pagination === null || pagination === void 0 ? void 0 : pagination.current) || TABLE_START_PAGE
      },
      filters: clearEmpty(defaultFiltersValue || {}),
      sorts: defaultSortsValue || [],
      search: clearEmpty(defaultSearchValue || {})
    }),
    loadParams = _b[0],
    setLoadParams = _b[1];
  /** 表格数据 */
  var _c = useState(data || []),
    tableData = _c[0],
    _setTableData = _c[1];
  /** 表格配置 */
  var ayTableFields = useMemo(function () {
    return getAyTableFields(fields, loadParams, tableData, _setTableData, ctrl, props);
  }, [fields, loadParams, tableData, _setTableData, ctrl, props]);
  var component = useMemo(function () {
    return getComponents(editMode);
  }, [editMode]);
  /** 是否正在加载 */
  var _d = useState(false),
    loading = _d[0],
    setLoading = _d[1];
  /** 总共多少条 */
  var _e = useState(0),
    total = _e[0],
    setTotal = _e[1];
  /** 禁用的选项的 key */
  var _f = useState([]),
    disabledKeys = _f[0],
    setDisabledKeys = _f[1];
  /** 拖拽单元格宽度的参数 */
  var _g = useAntdResizableHeader({
      columns: ayTableFields,
      cache: false,
      defaultWidth: 300
      // 保存拖拽宽度至本地localStorage
      // columnsState: {
      //   persistenceKey: 'localKey',
      //   persistenceType: 'localStorage',
      // },
    }),
    components = _g.components,
    resizableColumns = _g.resizableColumns,
    tableWidth = _g.tableWidth,
    resetColumns = _g.resetColumns;
  var resizableHeader = (tableExtend === null || tableExtend === void 0 ? void 0 : tableExtend.resizableHeader) || false;
  useEffect(function () {
    _setTableData(data || []);
  }, [data]);
  useEffect(function () {
    if (onParamsChange) {
      onParamsChange(loadParams);
    }
    if (initRef.current) {
      // 如果已经初始化，变动触发加载数据
      loadData();
    } else if (autoload !== false && !initRef.current) {
      // 尚未初始化 且 默认加载，触发加载数据
      loadData();
    }
  }, [loadParams]);
  useEffect(function () {
    initRef.current = true;
  }, []);
  /**
   * 获得查询前的参数
   */
  var getApiParams = function getApiParams() {
    var searchParams = __assign(__assign({}, loadParams), {
      search: __assign(__assign({}, extendSearchParams), loadParams.search)
    });
    // 默认筛选过滤
    if (defaultSearchFilter) {
      searchParams = defaultSearchFilter(searchParams);
    }
    // 主动筛选过滤
    if (beforeSearch) {
      searchParams = beforeSearch(searchParams);
    }
    return searchParams;
  };
  /**
   * 加载数据
   * @step 1、获得 params
   * @step 2、开始 loading
   * @step 3、加载数据
   * @step 4、设置表格数据
   * @step 5、关闭 loading
   */
  var loadData = function loadData() {
    if (api) {
      var searchParams = getApiParams();
      setLoading(true);
      api(searchParams).then(function (data) {
        data = defaultDataFilter(data);
        var content = data.content;
        if (filterData) {
          content = filterData(data);
        }
        _setTableData(content);
        setTotal(data.totalCount);
        if (onLoad) {
          onLoad(content, data);
        }
      }).finally(function () {
        setLoading(false);
      });
    }
    // eslint-disable-next-line react-hooks/exhaustive-deps
  };
  /**
   * 改变表格查询数据
   * @param pageSize 每页多少条
   * @param current 当前第几页
   * @param search 查询数据
   *
   * @param extraParams 额外的一个对象用来记录loaddata时候的操作
   */
  var updateLoadParams = function updateLoadParams(_a, extraParams) {
    var pageSize = _a.pageSize,
      current = _a.current,
      search = _a.search;
    var newLoadParams = __assign({}, loadParams);
    if (pageSize !== undefined) {
      newLoadParams.pagination.pageSize = pageSize;
    }
    if (current !== undefined) {
      newLoadParams.pagination.current = current;
    }
    if (search !== undefined) {
      newLoadParams.search = clearEmpty(search);
    }
    setLoadParams(__assign(__assign({}, newLoadParams), extraParams));
  };
  var handleTableChange = function handleTableChange(pagination, filters, sorter, extra) {
    var newParams = __assign({}, loadParams);
    if (getSearchParams) {
      newParams.search = clearEmpty(getSearchParams());
    }
    newParams.pagination = {
      pageSize: pagination.pageSize,
      current: pagination.current
    };
    // 获取过滤参数
    newParams.filters = {};
    for (var key in filters) {
      newParams.filters[key] = filters[key];
    }
    // 获取排序参数
    newParams.sorts = [];
    if (Array.isArray(sorter)) {
      sorter.forEach(function (option) {
        newParams.sorts.push({
          key: option.field,
          order: option.order
        });
      });
    } else if (sorter.field) {
      newParams.sorts = [{
        key: sorter.field,
        order: sorter.order
      }];
    }
    // 排序与筛选默认会回到第一页
    if (extra.action === 'sort' || extra.action === 'filter') {
      newParams.pagination.current = TABLE_START_PAGE;
      delete newParams._action;
    }
    setLoadParams(newParams);
  };
  useImperativeHandle(ref, function () {
    return {
      /**
       * 刷新页面
       * action暂时写在这里? 是否可以放到外一层的组件中传进来?
       */
      refresh: function refresh() {
        updateLoadParams({}, {
          _action: 'table'
        });
        // loadData()
      },

      /**
       * 回到第一页，刷新页面
       * action暂时写在这里? 是否可以放到外一层的组件中传进来?
       */
      reset: function reset(search) {
        updateLoadParams({
          search: search,
          current: TABLE_START_PAGE
        }, {
          _action: 'search'
        });
      },
      /**
       * 获取表格数据
       */
      getTableData: function getTableData() {
        return tableData;
      },
      /**
       * 设置表格数据
       */
      setTableData: function setTableData(tableData) {
        _setTableData(tableData);
      },
      /**
       * 根据 id 删除某一行数据
       */
      deleteRowByKey: function deleteRowByKey(key) {
        // @ts-ignore
        var newTableData = __spreadArray([], tableData, true);
        var index = newTableData.findIndex(function (row) {
          return getKey(row, rowKey) === key;
        });
        if (index >= 0) {
          newTableData.splice(index, 1);
          _setTableData(newTableData);
        }
      },
      /**
       * 插入一行数据
       * @param row 新增的数据
       * @param type 新增在前面还是后面
       */
      insertRow: function insertRow(row, key) {
        var newTableData = __spreadArray([], tableData, true);
        var index = newTableData.findIndex(function (row) {
          return getKey(row, rowKey) === key;
        });
        newTableData.splice(index + 1, 0, row);
        _setTableData(newTableData);
      },
      /**
       * 新增一行数据
       * @param row 新增的数据
       * @param type 新增在前面还是后面
       */
      addRow: function addRow(row, type) {
        if (type === void 0) {
          type = 'after';
        }
        var newTableData = __spreadArray([], tableData, true);
        newTableData[type === 'after' ? 'push' : 'unshift'](row);
        _setTableData(newTableData);
      },
      /**
       * 清空过滤
       * @param keys 要清空的 Filed 的 key 组成的数组
       */
      clearFilters: function clearFilters(keys) {
        if (keys === void 0) {
          keys = [];
        }
        var newParams = __assign({}, loadParams);
        for (var key in newParams.filters) {
          // 如果是空数组，则全部清除，如果是有数据的数组，则清空 key 相等的过滤值
          if (!keys.length || keys.includes(key)) {
            newParams.filters[key] = null;
          }
        }
        setLoadParams(newParams);
      },
      /**
       * 设置过滤值
       * @param filters { key: value } 组成的对象
       */
      setFiltersValue: function setFiltersValue(filters) {
        var newParams = __assign({}, loadParams);
        for (var key in filters) {
          newParams.filters[key] = filters[key];
        }
        setLoadParams(newParams);
      },
      /**
       * 清空排序
       * @param keys 要清空的 Filed 的 key 组成的数组
       */
      clearSorts: function clearSorts(keys) {
        if (keys === void 0) {
          keys = [];
        }
        var newParams = __assign({}, loadParams);
        if (!keys.length) {
          // 空数组则清空全部
          newParams.sorts = [];
        } else {
          for (var i = 0; i < newParams.sorts.length; i++) {
            var sortItem = newParams.sorts[i];
            // 清空 key 相等的 sorts 值
            if (keys.includes(sortItem.key)) {
              newParams.sorts.splice(i, 1);
              i -= 1;
            }
          }
        }
        setLoadParams(newParams);
      },
      /**
       * 设置排序值
       * @param sorts 排序的数据
       */
      setSortsValue: function setSortsValue(sorts) {
        var newParams = __assign({}, loadParams);
        newParams.sorts = sorts || [];
        setLoadParams(newParams);
      },
      getApiParams: getApiParams,
      /**
       * 更新分页值
       * @param paginition 分页数据
       */
      setPaginitionValue: function setPaginitionValue(paginition) {
        var newParams = __assign({}, loadParams);
        newParams.pagination = __assign(__assign({}, loadParams.pagination), paginition);
        setLoadParams(newParams);
      }
    };
  });
  useEffect(function () {
    _setTableData(data || []);
  }, [data]);
  /**
   * 是否拥有头部
   */
  var hasHeader = function hasHeader() {
    if (title || btnBefore) {
      return true;
    }
    // 检查是否拥有除了 MwFields 以外的子元素
    if (children) {
      var checkChildren = Array.isArray(children) ? __spreadArray([], children, true) : [children];
      // @ts-ignore
      if (checkChildren.filter(function (node) {
        var _a;
        return ((_a = node === null || node === void 0 ? void 0 : node.type) === null || _a === void 0 ? void 0 : _a.componentName) !== 'MwFields';
      }).length > 0) {
        return true;
      }
    }
    return false;
  };
  var paginationProps = __assign({
    showTotal: function showTotal(total) {
      return "".concat(locale.table.totalBefore, " ").concat(total, " ").concat(locale.table.totalAfter);
    },
    showQuickJumper: true,
    size: 'default',
    total: total,
    current: loadParams.pagination.current,
    pageSize: loadParams.pagination.pageSize
  }, pagination);
  return /*#__PURE__*/React.createElement(MwListContext.Provider, {
    value: {
      data: tableData,
      disabledKeys: disabledKeys,
      setDisabledKeys: setDisabledKeys
    }
  }, /*#__PURE__*/React.createElement(Card, {
    className: "mw-table ".concat(className || '')
  }, hasHeader() ? /*#__PURE__*/React.createElement("header", {
    className: "mw-table-header"
  }, /*#__PURE__*/React.createElement("div", {
    className: "mw-table-header-left"
  }, /*#__PURE__*/React.createElement(Space, null, typeof title === 'string' ? /*#__PURE__*/React.createElement("h2", {
    className: "mw-table-title"
  }, title) : title)), /*#__PURE__*/React.createElement("div", {
    className: "mw-table-header-right"
  }, /*#__PURE__*/React.createElement(Space, null, btnBefore, children))) : null, header, tableHeader, /*#__PURE__*/React.createElement("div", {
    className: "mw-table-content"
  }, /*#__PURE__*/React.createElement(Table, __assign({
    bordered: true,
    rowClassName: rowClassName,
    onExpand: onExpand,
    columns: resizableHeader ? resizableColumns : ayTableFields,
    components: resizableHeader ? __assign(__assign({}, component), components) : component,
    dataSource: tableData,
    loading: loading,
    rowSelection: rowSelection,
    pagination: pagination === false ? false : useOriginPagination === undefined ? paginationProps : false,
    onChange: handleTableChange,
    rowKey: rowKey,
    size: size,
    scroll: {
      x: resizableHeader ? tableWidth : scrollX,
      y: height
    }
  }, tableDefaultProps, tableExtend)), useOriginPagination === false && /*#__PURE__*/React.createElement(Pagination, __assign({
    className: "ant-table-pagination ant-table-pagination-right"
  }, paginationProps)))));
});