"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.install = void 0;
var _utils = require("../utils");
var _antd = require("antd");
var _react = _interopRequireDefault(require("react"));
var _MwSelect = _interopRequireDefault(require("../MwSelect"));
var _constant = require("../constant");
var _MwTagGroup = _interopRequireDefault(require("../MwTagGroup"));
var _MwCardGroup = _interopRequireDefault(require("../MwCardGroup"));
var _locale = _interopRequireDefault(require("../locale"));
function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }
var __assign = void 0 && (void 0).__assign || function () {
  __assign = Object.assign || function (t) {
    for (var s, i = 1, n = arguments.length; i < n; i++) {
      s = arguments[i];
      for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p)) t[p] = s[p];
    }
    return t;
  };
  return __assign.apply(this, arguments);
};
var install = function install(registerField) {
  // 注册输入框
  registerField(_constant.FORM_TYPE_INPUT, {
    type: _constant.FORM_TYPE_INPUT,
    defaultValue: _constant.FORM_DEFAULT_VALUE_INPUT,
    render: function render(_a) {
      var field = _a.field,
        readonly = _a.readonly,
        getFieldValue = _a.getFieldValue;
      return readonly ? /*#__PURE__*/_react.default.createElement("span", {
        className: "mw-form-text"
      }, getFieldValue(field.key) || _constant.FORM_READONLY_EMPTY) : /*#__PURE__*/_react.default.createElement(_antd.Input, __assign({
        placeholder: "".concat(_locale.default.form.pleaseInput).concat(field.title || '').concat(_locale.default.form.pleaseInputAfter),
        disabled: readonly,
        maxLength: _constant.INPUT_DEFAULT_MAXLENGTH,
        allowClear: _constant.FORM_DEFAULT_ALLOW_CLEAR
      }, field.props));
    }
  });
  // 注册搜索框
  registerField(_constant.FORM_TYPE_SEARCH, {
    type: _constant.FORM_TYPE_SEARCH,
    defaultValue: _constant.FORM_DEFAULT_VALUE_SEARCH,
    render: function render(_a) {
      var field = _a.field,
        readonly = _a.readonly,
        getFieldValue = _a.getFieldValue,
        formInstans = _a.formInstans;
      return readonly ? /*#__PURE__*/_react.default.createElement("span", {
        className: "mw-form-text"
      }, getFieldValue(field.key) || _constant.FORM_READONLY_EMPTY) : /*#__PURE__*/_react.default.createElement(_antd.Input.Search, __assign({
        placeholder: "".concat(_locale.default.form.pleaseInput).concat(field.title || '').concat(_locale.default.form.pleaseInputAfter),
        disabled: readonly,
        onPressEnter: function onPressEnter(e) {
          e.preventDefault();
          formInstans.submit();
        },
        onSearch: function onSearch() {
          return formInstans.submit();
        },
        maxLength: _constant.INPUT_DEFAULT_MAXLENGTH,
        allowClear: _constant.FORM_DEFAULT_ALLOW_CLEAR,
        enterButton: true
      }, field.props));
    }
  });
  // 注册数字框
  registerField(_constant.FORM_TYPE_NUMBER, {
    type: _constant.FORM_TYPE_NUMBER,
    defaultValue: _constant.FORM_DEFAULT_VALUE_NUMBER,
    render: function render(_a) {
      var field = _a.field,
        readonly = _a.readonly,
        getFieldValue = _a.getFieldValue;
      return readonly ? /*#__PURE__*/_react.default.createElement("span", {
        className: "mw-form-text"
      }, getFieldValue(field.key) || _constant.FORM_READONLY_EMPTY) : /*#__PURE__*/_react.default.createElement(_antd.InputNumber, __assign({
        className: "max-width",
        disabled: readonly,
        min: _constant.NUMBER_DEFAULT_MIN,
        max: _constant.NUMBER_DEFAULT_MAX
      }, field.props));
    }
  });
  // 注册百分比数字框
  registerField(_constant.FORM_TYPE_PERCENT, {
    type: _constant.FORM_TYPE_PERCENT,
    defaultValue: _constant.FORM_DEFAULT_VALUE_PERCENT,
    render: function render(_a) {
      var field = _a.field,
        readonly = _a.readonly,
        getFieldValue = _a.getFieldValue;
      return readonly ? /*#__PURE__*/_react.default.createElement("span", {
        className: "mw-form-text"
      }, getFieldValue(field.key) || _constant.FORM_READONLY_EMPTY, "%") : /*#__PURE__*/_react.default.createElement(_antd.InputNumber, __assign({
        className: "max-width",
        disabled: readonly,
        min: _constant.NUMBER_DEFAULT_MIN,
        max: _constant.PERCENT_DEFAULT_MAX,
        formatter: function formatter(value) {
          return value !== '' ? "".concat(value, "%") : '';
        },
        parser: function parser(value) {
          return (value || '').replace('%', '');
        }
      }, field.props));
    }
  });
  // 注册密码框
  registerField(_constant.FORM_TYPE_PASSWORD, {
    type: _constant.FORM_TYPE_PASSWORD,
    defaultValue: _constant.FORM_DEFAULT_VALUE_PASSWORD,
    render: function render(_a) {
      var field = _a.field,
        readonly = _a.readonly,
        getFieldValue = _a.getFieldValue;
      return readonly ? /*#__PURE__*/_react.default.createElement("span", {
        className: "mw-form-text"
      }, (getFieldValue(field.key) + '').replace(/./g, '*') || _constant.FORM_READONLY_EMPTY) : /*#__PURE__*/_react.default.createElement(_antd.Input.Password, __assign({
        placeholder: "".concat(_locale.default.form.pleaseInput).concat(field.title || '').concat(_locale.default.form.pleaseInputAfter),
        disabled: readonly,
        allowClear: _constant.FORM_DEFAULT_ALLOW_CLEAR
      }, field.props));
    }
  });
  // 注册多行文本框
  registerField(_constant.FORM_TYPE_TEXTAREA, {
    type: _constant.FORM_TYPE_TEXTAREA,
    defaultValue: _constant.FORM_DEFAULT_VALUE_TEXTAREA,
    render: function render(_a) {
      var field = _a.field,
        readonly = _a.readonly,
        getFieldValue = _a.getFieldValue;
      return readonly ? /*#__PURE__*/_react.default.createElement("span", {
        className: "mw-form-text"
      }, getFieldValue(field.key) || _constant.FORM_READONLY_EMPTY) : /*#__PURE__*/_react.default.createElement(_antd.Input.TextArea, __assign({
        placeholder: "".concat(_locale.default.form.pleaseInput).concat(field.title || '').concat(_locale.default.form.pleaseInputAfter),
        disabled: readonly,
        allowClear: _constant.FORM_DEFAULT_ALLOW_CLEAR,
        maxLength: _constant.TEXTAREA_DEFAULT_MAXLENGTH
      }, field.props));
    }
  });
  // 注册选择框
  registerField(_constant.FORM_TYPE_SELECT, {
    type: _constant.FORM_TYPE_SELECT,
    defaultValue: _constant.FORM_DEFAULT_VALUE_SELECT,
    render: function render(_a) {
      var field = _a.field,
        readonly = _a.readonly,
        getFieldValue = _a.getFieldValue;
      if (readonly) {
        var value = getFieldValue(field.key);
        var text = '';
        if (Array.isArray(value)) {
          if (!value.length) {
            text = _constant.FORM_READONLY_EMPTY;
          }
          text = value.map(function (item) {
            return (0, _utils.getValueByOptions)(item, field.options);
          }).join(field.splitText || '、');
        } else {
          text = (0, _utils.getValueByOptions)(value, field.options);
        }
        return /*#__PURE__*/_react.default.createElement("span", {
          className: "mw-form-text"
        }, text || _constant.FORM_READONLY_EMPTY);
      }
      return /*#__PURE__*/_react.default.createElement(_MwSelect.default, __assign({
        placeholder: "".concat(_locale.default.form.pleaseSelect).concat(field.title || '').concat(_locale.default.form.pleaseSelectAfter),
        disabled: readonly,
        allowClear: _constant.FORM_DEFAULT_ALLOW_CLEAR,
        options: field.options
      }, field.props));
    }
  });
  // 注册开关
  registerField(_constant.FORM_TYPE_SWITCH, {
    type: _constant.FORM_TYPE_SWITCH,
    defaultValue: _constant.FORM_DEFAULT_VALUE_SWITCH,
    valuePropName: 'checked',
    render: function render(_a) {
      var field = _a.field,
        readonly = _a.readonly;
      return /*#__PURE__*/_react.default.createElement(_antd.Switch, __assign({
        disabled: readonly
      }, field.props));
    }
  });
  // 注册单个选择框
  registerField(_constant.FORM_TYPE_CHECKBOX, {
    type: _constant.FORM_TYPE_CHECKBOX,
    defaultValue: _constant.FORM_DEFAULT_VALUE_CHECKBOX,
    valuePropName: 'checked',
    render: function render(_a) {
      var field = _a.field,
        readonly = _a.readonly;
      return /*#__PURE__*/_react.default.createElement(_antd.Checkbox, __assign({
        disabled: readonly
      }, field.props));
    }
  });
  // 注册多个选择框
  registerField(_constant.FORM_TYPE_CHECKBOX_GROUP, {
    type: _constant.FORM_TYPE_CHECKBOX_GROUP,
    defaultValue: _constant.FORM_DEFAULT_VALUE_CHECKBOX_GROUP,
    render: function render(_a) {
      var field = _a.field,
        readonly = _a.readonly;
      return /*#__PURE__*/_react.default.createElement(_antd.Checkbox.Group, __assign({
        disabled: readonly,
        options: field.options
      }, field.props));
    }
  });
  // 注册多个单选框
  registerField(_constant.FORM_TYPE_RADIO_GROUP, {
    type: _constant.FORM_TYPE_RADIO_GROUP,
    defaultValue: _constant.FORM_DEFAULT_VALUE_RADIO_GROUP,
    render: function render(_a) {
      var field = _a.field,
        readonly = _a.readonly;
      return /*#__PURE__*/_react.default.createElement(_antd.Radio.Group, __assign({
        disabled: readonly,
        options: field.options
      }, field.props));
    }
  });
  // 注册日期
  registerField(_constant.FORM_TYPE_DATE, {
    type: _constant.FORM_TYPE_DATE,
    defaultValue: _constant.FORM_DEFAULT_VALUE_DATE,
    render: function render(_a) {
      var field = _a.field,
        readonly = _a.readonly,
        getFieldValue = _a.getFieldValue;
      var text = getFieldValue(field.key, readonly);
      if (typeof text !== 'string') {
        text = '';
      }
      return readonly ? /*#__PURE__*/_react.default.createElement("span", {
        className: "mw-form-text"
      }, text || _constant.FORM_READONLY_EMPTY) : /*#__PURE__*/_react.default.createElement(_antd.DatePicker, __assign({
        className: "max-width",
        placeholder: "".concat(_locale.default.form.pleaseSelect).concat(field.title || '').concat(_locale.default.form.pleaseSelectAfter)
      }, field.props));
    }
  });
  // 注册区间日期
  registerField(_constant.FORM_TYPE_DATE_RANGE, {
    type: _constant.FORM_TYPE_DATE_RANGE,
    defaultValue: _constant.FORM_DEFAULT_VALUE_DATE_RANGE,
    render: function render(_a) {
      var field = _a.field,
        readonly = _a.readonly,
        getFieldValue = _a.getFieldValue;
      var text = getFieldValue(field.key, readonly);
      if (Array.isArray(text)) {
        if (text[0] === null) {
          text = null;
        } else if (text) {
          text = [/*#__PURE__*/_react.default.createElement("span", {
            key: "start",
            style: {
              display: 'inline-block'
            }
          }, (text[0] || '').toString()), /*#__PURE__*/_react.default.createElement("span", {
            key: "divider",
            style: {
              margin: '0 0.5em'
            }
          }, _locale.default.form.dateTo), /*#__PURE__*/_react.default.createElement("span", {
            key: "end",
            style: {
              display: 'inline-block'
            }
          }, (text[1] || '').toString())];
        }
      }
      return readonly ? /*#__PURE__*/_react.default.createElement("span", {
        className: "mw-form-text"
      }, text || _constant.FORM_READONLY_EMPTY) : /*#__PURE__*/_react.default.createElement(_antd.DatePicker.RangePicker, __assign({
        placeholder: [_locale.default.form.startDate, _locale.default.form.endDate],
        className: "max-width"
      }, field.props));
    }
  });
  // 注册空节点
  registerField(_constant.FORM_TYPE_EMPTY, {
    type: _constant.FORM_TYPE_EMPTY,
    defaultValue: _constant.FORM_DEFAULT_VALUE_EMPTY,
    render: function render() {
      return /*#__PURE__*/_react.default.createElement("input", {
        hidden: true,
        type: "text"
      });
    }
  });
  // 注册评分
  registerField(_constant.FORM_TYPE_RATE, {
    type: _constant.FORM_TYPE_RATE,
    defaultValue: _constant.FORM_DEFAULT_VALUE_RATE,
    render: function render(_a) {
      var field = _a.field,
        readonly = _a.readonly;
      return /*#__PURE__*/_react.default.createElement(_antd.Rate, __assign({
        disabled: readonly
      }, field.props));
    }
  });
  // 滑动输入条
  registerField(_constant.FORM_TYPE_SLIDER, {
    type: _constant.FORM_TYPE_SLIDER,
    defaultValue: _constant.FORM_DEFAULT_VALUE_SLIDER,
    render: function render(_a) {
      var field = _a.field,
        readonly = _a.readonly;
      return /*#__PURE__*/_react.default.createElement(_antd.Slider, __assign({
        disabled: readonly
      }, field.props));
    }
  });
  // 标签选择
  registerField(_constant.FORM_TYPE_TAG_GROUP, {
    type: _constant.FORM_TYPE_TAG_GROUP,
    defaultValue: _constant.FORM_DEFAULT_VALUE_TAG_GROUP,
    render: function render(_a) {
      var field = _a.field,
        readonly = _a.readonly;
      return /*#__PURE__*/_react.default.createElement(_MwTagGroup.default, __assign({
        readonly: readonly,
        options: field.options
      }, field.props));
    }
  });
  // 卡片选择
  registerField(_constant.FORM_TYPE_CARD_GROUP, {
    type: _constant.FORM_TYPE_CARD_GROUP,
    defaultValue: _constant.FORM_DEFAULT_VALUE_CARD_GROUP,
    render: function render(_a) {
      var field = _a.field,
        readonly = _a.readonly;
      return /*#__PURE__*/_react.default.createElement(_MwCardGroup.default, __assign({
        readonly: readonly,
        options: field.options
      }, field.props));
    }
  });
  // 注册 html
  registerField(_constant.FORM_TYPE_HTML, {
    type: _constant.FORM_TYPE_HTML,
    defaultValue: _constant.FORM_DEFAULT_VALUE_HTML,
    render: function render(_a) {
      var field = _a.field;
      return /*#__PURE__*/_react.default.createElement("div", {
        dangerouslySetInnerHTML: {
          __html: field.html
        }
      });
    }
  });
};
exports.install = install;