"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.registerField = exports.getFieldDefaultValue = exports.getDefaultValue = exports.funcs = exports.default = void 0;
var _react = _interopRequireWildcard(require("react"));
var _MwCard = _interopRequireDefault(require("../MwCard"));
var _Theme = require("../Theme");
var _antd = require("antd");
var _utils = require("../utils");
var _FieldsInit = require("./FieldsInit");
var _constant = require("../constant");
var _moment = _interopRequireDefault(require("moment"));
require("moment/locale/zh-cn");
var _convertFields = require("../MwFields/convertFields");
var _icons = require("@ant-design/icons");
var _locale = _interopRequireDefault(require("../locale"));
var _parseFields = _interopRequireWildcard(require("./parseFields"));
var _MwFormList = _interopRequireDefault(require("./MwFormList"));
function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }
function _getRequireWildcardCache(nodeInterop) { if (typeof WeakMap !== "function") return null; var cacheBabelInterop = new WeakMap(); var cacheNodeInterop = new WeakMap(); return (_getRequireWildcardCache = function _getRequireWildcardCache(nodeInterop) { return nodeInterop ? cacheNodeInterop : cacheBabelInterop; })(nodeInterop); }
function _interopRequireWildcard(obj, nodeInterop) { if (!nodeInterop && obj && obj.__esModule) { return obj; } if (obj === null || _typeof(obj) !== "object" && typeof obj !== "function") { return { default: obj }; } var cache = _getRequireWildcardCache(nodeInterop); if (cache && cache.has(obj)) { return cache.get(obj); } var newObj = {}; var hasPropertyDescriptor = Object.defineProperty && Object.getOwnPropertyDescriptor; for (var key in obj) { if (key !== "default" && Object.prototype.hasOwnProperty.call(obj, key)) { var desc = hasPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : null; if (desc && (desc.get || desc.set)) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } newObj.default = obj; if (cache) { cache.set(obj, newObj); } return newObj; }
function _typeof(obj) { "@babel/helpers - typeof"; return _typeof = "function" == typeof Symbol && "symbol" == typeof Symbol.iterator ? function (obj) { return typeof obj; } : function (obj) { return obj && "function" == typeof Symbol && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; }, _typeof(obj); }
var __assign = void 0 && (void 0).__assign || function () {
  __assign = Object.assign || function (t) {
    for (var s, i = 1, n = arguments.length; i < n; i++) {
      s = arguments[i];
      for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p)) t[p] = s[p];
    }
    return t;
  };
  return __assign.apply(this, arguments);
};
var __rest = void 0 && (void 0).__rest || function (s, e) {
  var t = {};
  for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p) && e.indexOf(p) < 0) t[p] = s[p];
  if (s != null && typeof Object.getOwnPropertySymbols === "function") for (var i = 0, p = Object.getOwnPropertySymbols(s); i < p.length; i++) {
    if (e.indexOf(p[i]) < 0 && Object.prototype.propertyIsEnumerable.call(s, p[i])) t[p[i]] = s[p[i]];
  }
  return t;
};
var __spreadArray = void 0 && (void 0).__spreadArray || function (to, from, pack) {
  if (pack || arguments.length === 2) for (var i = 0, l = from.length, ar; i < l; i++) {
    if (ar || !(i in from)) {
      if (!ar) ar = Array.prototype.slice.call(from, 0, i);
      ar[i] = from[i];
    }
  }
  return to.concat(ar || Array.prototype.slice.call(from));
};
_moment.default.locale('zh-cn');
var defaultLayout = {
  labelCol: {
    flex: '120px'
  },
  wrapperCol: {
    flex: '1'
  }
};
var fieldMap = {};
/**
 * 注册一个 field type
 * @param fieldType field 类型
 * @param field 注册的 field
 */
var registerField = function registerField(fieldType, field) {
  fieldMap[fieldType] = field;
};
// 初始化注册 field
exports.registerField = registerField;
(0, _FieldsInit.install)(registerField);
/**
 * 获取隐藏配置项
 * @param field 配置项
 */
var getNoVisibleField = function getNoVisibleField(field) {
  return __assign(__assign({}, field), {
    title: '',
    type: 'empty'
  });
};
/**
 * 生成 placeholder
 * @param field 配置项
 */
var getPlaceholder = function getPlaceholder(field) {
  var defaultProps = field.props;
  if (defaultProps && defaultProps.placeholder) {
    return defaultProps.placeholder;
  }
  if (!field.type) {
    return "".concat(_locale.default.form.pleaseInput).concat(field.title || '').concat(_locale.default.form.pleaseInputAfter);
  }
  if ([_constant.FORM_TYPE_INPUT, _constant.FORM_TYPE_NUMBER, _constant.FORM_TYPE_PERCENT, _constant.FORM_TYPE_PASSWORD, _constant.FORM_TYPE_TEXTAREA].includes(field.type)) {
    return "".concat(_locale.default.form.pleaseInput).concat(field.title || '').concat(_locale.default.form.pleaseInputAfter);
  } else if ([_constant.FORM_TYPE_SELECT, _constant.FORM_TYPE_DATE, _constant.FORM_TYPE_DATE_RANGE, _constant.FORM_TYPE_CHECKBOX, _constant.FORM_TYPE_CHECKBOX_GROUP, _constant.FORM_TYPE_RADIO_GROUP, _constant.FORM_TYPE_CARD_GROUP, _constant.FORM_TYPE_TAG_GROUP].includes(field.type)) {
    return "".concat(_locale.default.form.pleaseSelect).concat(field.title || '').concat(_locale.default.form.pleaseSelectAfter);
  }
  return _locale.default.form.pleaseInput + field.title || '';
};
/**
 * 获得配置列表
 * @param fields 配置列表
 */
var getDefaultValue = function getDefaultValue(fields) {
  var form = {};
  fields.forEach(function (field) {
    var _a;
    if ([_constant.FORM_TYPE_CARD, _constant.FORM_TYPE_GROUP, _constant.FORM_TYPE_INPUT_GROUP].includes(field.type || '')) {
      var children = field.children || [];
      if (!Array.isArray(children)) {
        children = [children];
      }
      form = __assign(__assign({}, form), getDefaultValue(children));
      return;
    }
    if (field.type === _constant.FORM_TYPE_LIST) {
      form = __assign(__assign({}, form), (_a = {}, _a[field.key || ''] = __spreadArray([], field.defaultValue || [], true), _a));
      return;
    }
    var type = field.type || 'input';
    var key = (field === null || field === void 0 ? void 0 : field.key) || '';
    // 如果配置项里存在默认值，直接返回默认值，否则从默认值表里获取
    if (field.hasOwnProperty('defaultValue')) {
      // 日期类型的需要通过 moment 转一遍
      if (type === _constant.FORM_TYPE_DATE && field.defaultValue) {
        form[key] = (0, _moment.default)(field.defaultValue);
      } else if (type === _constant.FORM_TYPE_DATE_RANGE && field.defaultValue) {
        var _b = field.defaultValue,
          value0 = _b[0],
          value1 = _b[1];
        form[key] = [value0 ? (0, _moment.default)(value0) : null, value1 ? (0, _moment.default)(value1) : null];
      } else {
        form[key] = field.defaultValue;
      }
    } else if (type) {
      if (fieldMap[type]) {
        var fieldItem = fieldMap[type];
        var defaultValue = fieldItem.defaultValue;
        defaultValue = _typeof(defaultValue) === 'object' ? (0, _utils.copy)(defaultValue) : defaultValue;
        // 如果是多选则默认值是数组
        if (field.multiple || field.mode === 'multiple') {
          form[key] = [];
        } else {
          form[key] = defaultValue;
        }
      } else {
        form[key] = undefined;
      }
    }
  });
  return form;
};
exports.getDefaultValue = getDefaultValue;
var getFieldDefaultValue = function getFieldDefaultValue(key, fields) {
  if (!key) {
    return '';
  }
  var field = getField(key, fields);
  if (field) {
    var type = field.type || 'input';
    // 如果配置项里存在默认值，直接返回默认值，否则从默认值表里获取
    if (field.hasOwnProperty('defaultValue')) {
      return field.defaultValue;
    } else if (type) {
      if (fieldMap[type]) {
        var fieldItem = fieldMap[type];
        var defaultValue = fieldItem.defaultValue;
        defaultValue = _typeof(defaultValue) === 'object' ? (0, _utils.copy)(defaultValue) : defaultValue;
        return defaultValue;
      } else {
        return '';
      }
    }
  }
};
exports.getFieldDefaultValue = getFieldDefaultValue;
var fieldKeys = ['props', 'title', 'key', 'options', 'type', 'grid', 'span', 'defaultValue', 'order', 'required', 'rules', 'visible', 'hidden', 'formItemProps', 'renderContent', 'onChange', 'help', 'startKey', 'endKey', 'formatRule', 'readonlyFormatRule', 'reSetting', 'tooltip'];
var usedKeys = ['readonly', 'hiddenMode', 'required', 'table', 'search', 'dialog', 'sort', 'filter', 'filterMultiple', 'renderType', 'editable', 'before', 'after', 'defaultValue'];
/**
 * 根据不同的 type 生成不同种类的标签 Tag
 * @param field 配置项
 */
var getTag = function getTag(field, fields, formInstans, readonly, childrenType) {
  var type = field.type;
  type = type || 'input';
  var tag = null;
  var newField = __assign({}, field);
  // 移除无需添加的数据
  newField.props = (0, _utils.omitObj)(newField, usedKeys);
  if (fieldMap[type || '']) {
    var fieldItem = fieldMap[type || ''];
    tag = fieldItem.render({
      field: newField,
      setFieldsValue: formInstans.setFieldsValue,
      formInstans: formInstans,
      readonly: readonly || field.readonly || false,
      getFieldValue: childrenType === 'list' ? function () {
        return field.text;
      } : formInstans.getFieldValue
    });
  } else {
    switch (type) {
      case _constant.FORM_TYPE_CUSTOM:
        if (typeof field.renderContent === 'function') {
          tag = field.renderContent(newField, formInstans.getFieldsValue() || getDefaultValue(fields));
        }
        break;
    }
  }
  return tag;
};
/**
 * 通过配置列表转 Form.Item
 * @param fields 配置列表
 * @param formInstans form 实例
 * @param props form 的 props
 * @param childrenType 子类型
 */
var getFormItem = function getFormItem(fields, formInstans, props, childrenType) {
  var span = props.span,
    readonly = props.readonly,
    formLayout = props.formLayout,
    gutter = props.gutter;
  var ayFormProps = props;
  return fields.map(function (field, index) {
    // 把其它属性 添加到 props 里面
    field = __assign(__assign({}, field), {
      props: __assign(__assign({}, (0, _utils.omitObj)(field, fieldKeys)), field.props)
    });
    var fieldSpan = field.span !== 0 ? field.span || span || 24 : span || 24;
    if (field.type === _constant.FORM_TYPE_CARD) {
      var children = field.children || [];
      if (!Array.isArray(children)) {
        children = [children];
      }
      var content_1 = getFormItem(children, formInstans, ayFormProps, _constant.FORM_TYPE_CARD);
      return /*#__PURE__*/_react.default.createElement(_antd.Col, {
        key: field.key,
        span: field.span || 24
      }, /*#__PURE__*/_react.default.createElement(_MwCard.default, __assign({
        title: field.title
      }, field.props), /*#__PURE__*/_react.default.createElement(_antd.Row, {
        gutter: gutter
      }, content_1)));
    }
    var visible = true;
    // 隐藏该项目，保留占位，但是保留值
    if (field.visible !== undefined) {
      visible = typeof field.visible === 'function' ? field.visible() : field.visible;
    }
    var hidden = false;
    // 隐藏该项目，不保留占位，但是保留值
    if (field.hidden !== undefined) {
      hidden = typeof field.hidden === 'function' ? field.hidden() : field.hidden;
    }
    // 隐藏该项，只显示占位，保留 form 值
    if (!visible || hidden) {
      field = getNoVisibleField(field);
    }
    // 设置 Form.Item 的属性
    var formItemProps = __assign(__assign({}, field.formItemProps), {
      label: field.title,
      name: field.key,
      extra: field.help
    });
    // 如果自元素类型是列表，则重置 name，此时一定有 field.formItemProps
    if (childrenType === _constant.FORM_TYPE_LIST) {
      formItemProps.name = field.formItemProps.name;
    }
    // 组合元素的 formItem 不需要样式
    if (childrenType === _constant.FORM_TYPE_GROUP || childrenType === _constant.FORM_TYPE_INPUT_GROUP) {
      formItemProps.noStyle = true;
    }
    // 设定 开关、多选框 等的值类型 （这是 ant design form 的限制）
    if (field.type && fieldMap[field.type]) {
      formItemProps.valuePropName = fieldMap[field.type].valuePropName || 'value';
    }
    // 设置每个【表单项】的占位
    var colProps = {
      span: fieldSpan,
      offset: field.offset
    };
    // 填充 rules 属性
    if (field.rules) {
      formItemProps.rules = __spreadArray([], field.rules, true);
    }
    // 填充快捷 required 属性
    if (field.required) {
      var rule = {
        required: true,
        message: getPlaceholder(field) + _locale.default.form.requiredText
      };
      if (field.children && field.type !== _constant.FORM_TYPE_CHECKBOX) {
        formItemProps.label = /*#__PURE__*/_react.default.createElement("span", null, /*#__PURE__*/_react.default.createElement("span", {
          className: "required-mark"
        }, "*"), field.title);
      } else {
        if (formItemProps.rules) {
          formItemProps.rules.push(rule);
        } else {
          formItemProps.rules = [rule];
        }
      }
    }
    // 不保留占位
    if (hidden) {
      colProps.span = 0;
      colProps.xs = 0;
      colProps.sm = 0;
      colProps.md = 0;
      colProps.lg = 0;
      colProps.xl = 0;
    }
    // 不显示状态下 rule 无效
    if (hidden || !visible) {
      formItemProps.rules = [];
    }
    // 支持 tooltip 属性
    if (field.tooltip) {
      formItemProps.label = /*#__PURE__*/_react.default.createElement("span", null, field.title, /*#__PURE__*/_react.default.createElement(_antd.Tooltip, {
        placement: "top",
        title: field.tooltip
      }, /*#__PURE__*/_react.default.createElement(_icons.QuestionCircleOutlined, {
        style: {
          marginLeft: 4
        }
      })));
    }
    var tag;
    switch (field.type) {
      // 组合类型
      case _constant.FORM_TYPE_GROUP:
        tag = /*#__PURE__*/_react.default.createElement(_antd.Row, __assign({
          className: "mw-form-group"
        }, field.props), getFormItem(field.children, formInstans, ayFormProps, _constant.FORM_TYPE_GROUP));
        break;
      // 输入框组合
      case _constant.FORM_TYPE_INPUT_GROUP:
        tag = /*#__PURE__*/_react.default.createElement(_antd.Input.Group, __assign({
          compact: true
        }, field.props), getFormItem(field.children, formInstans, ayFormProps, _constant.FORM_TYPE_INPUT_GROUP));
        break;
      // 列表类型
      case _constant.FORM_TYPE_LIST:
        tag = /*#__PURE__*/_react.default.createElement(_MwFormList.default, {
          field: field,
          formInstant: formInstans,
          getFormItem: getFormItem,
          ayFormProps: ayFormProps
        });
        break;
      default:
        tag = getTag(field, fields, formInstans, readonly, childrenType);
        break;
    }
    // const content = field.render ? (
    //   field.render(field as MwFormField, formInstans.getFieldsValue() || getDefaultValue(fields), index)
    // ) : (
    //   <Form.Item key={field.key} {...formItemProps}>
    //     {tag}
    //   </Form.Item>
    // )
    var content = /*#__PURE__*/_react.default.createElement(_antd.Form.Item, __assign({
      key: field.key
    }, formItemProps), tag);
    if (formLayout === 'inline' || childrenType === _constant.FORM_TYPE_INPUT_GROUP) {
      return content;
    }
    return /*#__PURE__*/_react.default.createElement(_antd.Col, __assign({
      key: field.key
    }, colProps), content);
  });
};
var getField = function getField(key, fields) {
  var field = null;
  var loop = function loop(fields) {
    for (var i = 0; i < fields.length; i++) {
      var item = fields[i];
      if (item.key === key) {
        field = item;
        break;
      } else if (Array.isArray(item.children) && item.children.length) {
        loop(item.children);
      }
    }
  };
  loop(fields);
  return field;
};
/**
 * 格式化 日期
 * @param values 格式化的数据
 * @param fields 配置项
 */
var formatValues = function formatValues(values, fields) {
  var _a, _b, _c;
  var result = {};
  for (var key in values) {
    if (key.startsWith('__')) {
      continue;
    }
    var value = values[key];
    var field = getField(key, fields);
    if (value && field) {
      // 获得格式化日期格式
      var formatRule = (field === null || field === void 0 ? void 0 : field.showTime) || ((_a = field === null || field === void 0 ? void 0 : field.props) === null || _a === void 0 ? void 0 : _a.showTime) ? 'YYYY-MM-DD HH:mm:ss' : 'YYYY-MM-DD';
      if (field.formatRule) {
        formatRule = field.formatRule;
      }
      if (Array.isArray(value) && field.type === _constant.FORM_TYPE_DATE_RANGE) {
        // 区间类型取 startKey 与 endKey
        if (value[0]) {
          if (typeof value[0] === 'string') {
            result[field.startKey || 'startDate'] = value[0];
          } else {
            result[field.startKey || 'startDate'] = ((_b = value[0]) === null || _b === void 0 ? void 0 : _b.format(formatRule)) || null;
          }
        }
        if (value[1]) {
          if (typeof value[1] === 'string') {
            result[field.endKey || 'endDate'] = value[1];
          } else {
            result[field.endKey || 'endDate'] = ((_c = value[1]) === null || _c === void 0 ? void 0 : _c.format(formatRule)) || null;
          }
        }
      } else if (field.type === _constant.FORM_TYPE_DATE) {
        // 单值类型直接转
        if (typeof value === 'string') {
          result[key] = value;
        } else {
          result[key] = (value === null || value === void 0 ? void 0 : value.format(formatRule)) || null;
        }
      } else {
        result[key] = value;
      }
    } else {
      result[key] = value;
      // undefined 视为 null
      if (value === undefined) {
        result[key] = null;
      }
    }
  }
  return result;
};
/**
 * 提交表单，如果有 onConfirm 事件传入，则触发一次
 * @param values 表单值
 * @param onConfirm 提交表单事件
 * @param onFinish 提交表单事件，效果跟 onConfirm 一致
 * @param onSubmit 提交表单事件，效果跟 onConfirm 一致
 */
var handleConfirm = function handleConfirm(values, fields, onConfirm, onFinish, onSubmit) {
  var filterValues = formatValues(values, fields);
  if (onConfirm) {
    onConfirm(filterValues);
  }
  if (onFinish) {
    onFinish(filterValues);
  }
  if (onSubmit) {
    onSubmit(filterValues);
  }
};
/**
 * 支持表单改变事件监听
 * @param changedValues 改变的值
 * @param allValues 表单所有的值
 * @param fields 所有的饿配置项
 * @param setFieldsValue 设置表单值的方法
 */
var handleChange = function handleChange(changedValues, allValues, fields, setFieldsValue) {
  for (var key in changedValues) {
    var field = getField(key, fields);
    if (field) {
      var value = changedValues[key];
      if (field.onChange) {
        field.onChange(value, allValues, setFieldsValue);
      }
    }
  }
};
/**
 * 获取 MwForm 样式
 * @param className 外部 className
 * @param desc 是否处于文档模式
 * @param readonly 是否处于只读模式
 * @param clear 只读模式是否取消背景
 */
var getAyFormClassName = function getAyFormClassName(className, desc, readonly, noBackground) {
  var classList = ['mw-form', _Theme.theme];
  if (className) {
    classList.push(className);
  }
  if (desc) {
    classList.push('desc');
  }
  if (readonly) {
    classList.push('readonly');
  }
  if (noBackground) {
    classList.push('mw-form-no-background');
  }
  return classList.join(' ');
};
/**
 * antd form 原生支持的方法尽数暴露出去
 */
var funcs = ['getFieldValue', 'getFieldsValue', 'getFieldError', 'getFieldsError', 'isFieldTouched', 'isFieldsTouched', 'isFieldValidating', 'resetFields', 'scrollToField', 'setFields', 'setFieldsValue', 'submit', 'validateFields'];
exports.funcs = funcs;
var _default = /*#__PURE__*/(0, _react.forwardRef)(function MwForm(props, ref) {
  var originFields = props.fields,
    _a = props.formLayout,
    formLayout = _a === void 0 ? 'horizontal' : _a,
    onConfirm = props.onConfirm,
    _onFinish = props.onFinish,
    onSubmit = props.onSubmit,
    children = props.children,
    defaultProps = props.props,
    readonly = props.readonly,
    desc = props.desc,
    noBackground = props.noBackground,
    layout = props.layout,
    className = props.className,
    style = props.style,
    labelAlign = props.labelAlign,
    gutter = props.gutter,
    otherProps = __rest(props
    // 子元素转化出来的 fields + 标签上的 fields
    , ["fields", "formLayout", "onConfirm", "onFinish", "onSubmit", "children", "props", "readonly", "desc", "noBackground", "layout", "className", "style", "labelAlign", "gutter"]);
  // 子元素转化出来的 fields + 标签上的 fields
  var totalFields = (0, _react.useMemo)(function () {
    var childrenFields = (0, _convertFields.convertChildrenToField)(children);
    return __spreadArray(__spreadArray([], originFields || [], true), childrenFields, true);
  }, [originFields, children]);
  /** 当前表单值 */
  var _b = (0, _react.useState)(getDefaultValue(totalFields)),
    formValues = _b[0],
    setFormValues = _b[1];
  /** 转换过后的 fields */
  var _c = (0, _react.useState)((0, _parseFields.default)(totalFields, formValues)),
    fields = _c[0],
    setFields = _c[1];
  /** 是否初始化 */
  var _d = (0, _react.useState)(false),
    firstMount = _d[0],
    setFirstMount = _d[1];
  /** 控制 any form 的实例 */
  var formRef = (0, _react.useRef)();
  /** 暴露出去的 form 的实例，允许父组件通过 ref 调用方法 */
  var formInstans = {};
  (0, _react.useEffect)(function () {
    if (!firstMount) {
      setFields((0, _parseFields.default)(totalFields, formValues));
    } else {
      setFirstMount(true);
    }
  }, [totalFields, formValues]);
  /** 填充方法 */
  funcs.forEach(function (func) {
    formInstans[func] = function () {
      var _a;
      var args = [];
      for (var _i = 0; _i < arguments.length; _i++) {
        args[_i] = arguments[_i];
      }
      return (_a = formRef.current)[func].apply(_a, args);
    };
  });
  // 设置表单值
  formInstans.setFieldsValue = function (values) {
    fields.forEach(function (field) {
      try {
        var key = (field === null || field === void 0 ? void 0 : field.key) || '';
        var value = values[key];
        if (field.type === _constant.FORM_TYPE_DATE) {
          if (value) {
            values[key] = (0, _moment.default)(value);
          }
        } else if (field.type === _constant.FORM_TYPE_DATE_RANGE && Array.isArray(value)) {
          var _a = value,
            value0 = _a[0],
            value1 = _a[1];
          values[key] = [value0 ? (0, _moment.default)(value0) : null, value1 ? (0, _moment.default)(value1) : null];
        }
      } catch (_b) {
        console.error('日期值转化错误');
      }
    });
    formRef.current.setFieldsValue(values);
    var newFormValues = __assign(__assign({}, getFieldsValue()), values);
    setFormValues(newFormValues);
  };
  // 刷新 field
  formInstans.refreshFields = function () {
    setFields((0, _parseFields.default)(totalFields, formValues));
  };
  /**
   * 获取 field 的值
   * 如果 field 还没有渲染完，那获得的是 defaultValue
   * @param key field 的 key
   * @param readonly 是否只读
   */
  var getFieldValue = function getFieldValue(key, readonly) {
    var value = getFieldDefaultValue(key, fields);
    if (formRef.current) {
      value = formRef.current.getFieldValue(key);
    }
    var field = getField(key, fields);
    if (field && value) {
      // 日期或者日期区间
      if (field.type === _constant.FORM_TYPE_DATE || Array.isArray(value) && field.type === _constant.FORM_TYPE_DATE_RANGE) {
        value = (0, _parseFields.getDateValue)(value, field, readonly) || null;
      }
    }
    return value;
  };
  /**
   * 根据 fields 获取 values
   * @param fields 配置项
   * @param readonly 是否只读
   * @returns object
   */
  var getValues = function getValues(fields, readonly) {
    var result = {};
    fields === null || fields === void 0 ? void 0 : fields.forEach(function (field) {
      if (!field.key) {
        return;
      }
      // 获取每个单个的值
      var value = getFieldValue(field.key, readonly);
      // 处理子层数据
      if (field.children && field.children.length && Array.isArray(field.children)) {
        var values = getValues(field.children, readonly);
        result = __assign(__assign({}, result), values);
      }
      result[field.key] = value;
    });
    return result;
  };
  /**
   * 获取所有 field 的值
   * @param readonly 是否只读
   */
  var getFieldsValue = function getFieldsValue(readonly) {
    return formatValues(getValues(fields, readonly), fields);
  };
  /**
   * 重置表单值
   */
  var resetFields = function resetFields() {
    formRef.current.resetFields();
    setFormValues(getDefaultValue(totalFields));
  };
  /** 覆盖 antd Form resetFields 方法 */
  formInstans.resetFields = resetFields;
  /** 覆盖 antd Form getFieldsValue 方法 */
  formInstans.getFieldsValue = getFieldsValue;
  /** 覆盖 antd Form getFieldValue 方法 */
  formInstans.getFieldValue = getFieldValue;
  /** 添加 getFormatFiledsValue 的值，历史遗留 */
  formInstans.getFormatFieldsValue = getFieldsValue;
  /** 暴露方法 */
  (0, _react.useImperativeHandle)(ref, function () {
    return formInstans;
  });
  var formItemLayout = formLayout === 'horizontal' ? __assign(__assign({}, defaultLayout), layout) : null;
  return /*#__PURE__*/_react.default.createElement("div", {
    className: getAyFormClassName(className, desc, readonly, noBackground),
    style: style
  }, /*#__PURE__*/_react.default.createElement(_antd.Form, __assign({
    ref: formRef,
    colon: desc ? false : true,
    layout: formLayout,
    labelAlign: labelAlign,
    labelWrap: true
  }, formItemLayout, otherProps, defaultProps, {
    initialValues: getDefaultValue(fields),
    onFinish: function onFinish(values) {
      return handleConfirm(values, fields, onConfirm, _onFinish, onSubmit);
    },
    onValuesChange: function onValuesChange(changedValues, allValues) {
      setFormValues(formatValues(allValues, fields));
      handleChange(changedValues, allValues, fields, formInstans.setFieldsValue);
    }
  }), /*#__PURE__*/_react.default.createElement(_antd.Row, {
    gutter: gutter
  }, getFormItem(fields, formInstans, props), children)));
});
exports.default = _default;