"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = parseFields;
exports.isExpression = exports.getDateValue = void 0;
exports.parseExpression = parseExpression;
var _constant = require("../constant");
var _moment = _interopRequireDefault(require("moment"));
var _react = require("react");
var _utils = require("../utils");
function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }
var __assign = void 0 && (void 0).__assign || function () {
  __assign = Object.assign || function (t) {
    for (var s, i = 1, n = arguments.length; i < n; i++) {
      s = arguments[i];
      for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p)) t[p] = s[p];
    }
    return t;
  };
  return __assign.apply(this, arguments);
};
/** 是否是表达式 */
var isExpression = function isExpression(value) {
  if (typeof value !== 'string') {
    return false;
  }
  return /^{{(.+)}}$/.test(value);
};
/** 转化表达式 */
exports.isExpression = isExpression;
function parseExpression(expression, formValues) {
  var func = expression.substring(2, expression.length - 2);
  var str = func.replace(/formValues/g, JSON.stringify(formValues));
  try {
    return new Function('return (' + str + ')')();
  } catch (_a) {
    console.error('fields 表达式转化错误');
  }
  return str;
}
/**
 * 转化表单项了里面所有的表达式
 * @param fields 当前配置项
 * @param formatValues 表单的值
 * @returns 新的配置项
 */
function parseFields(fields, formatValues) {
  var loop = function loop(field) {
    var newField = field;
    for (var key in newField) {
      var value = newField[key];
      if (Array.isArray(value) && key === 'children') {
        // 携带子元素
        newField[key] = value.map(function (field) {
          return loop(__assign({}, field));
        });
      } else if ((0, _utils.isObj)(value)) {
        // 过滤掉 moment 方法、ReactElement 节点
        if (_moment.default.isMoment(value) || /*#__PURE__*/(0, _react.isValidElement)(value)) {
          continue;
        }
        newField[key] = loop(__assign({}, value));
      } else if (isExpression(value)) {
        // 把表达式转化成值
        newField[key] = parseExpression(value, formatValues);
      }
    }
    return newField;
  };
  return fields.map(function (field) {
    return loop(__assign({}, field));
  });
}
/**
 * 获得格式化后的日期
 * @param value 当前值
 * @param field 配置项
 * @param readonly 是否只读
 * @returns
 */
var getDateValue = function getDateValue(value, field, readonly) {
  var _a;
  // 获得格式化日期格式
  var formatRule = (field === null || field === void 0 ? void 0 : field.showTime) || ((_a = field === null || field === void 0 ? void 0 : field.props) === null || _a === void 0 ? void 0 : _a.showTime) ? 'YYYY-MM-DD HH:mm:ss' : 'YYYY-MM-DD';
  if (field.formatRule) {
    formatRule = field.formatRule;
  }
  // 只读模式下，格式化日期取 readonlyFormatRule
  if (field.readonlyFormatRule && readonly) {
    formatRule = field.readonlyFormatRule;
  }
  if (field.type === _constant.FORM_TYPE_DATE) {
    // 日期格式化
    value = value ? (0, _moment.default)(value).format(formatRule) : null;
  } else if (Array.isArray(value) && field.type === _constant.FORM_TYPE_DATE_RANGE) {
    var value0 = value[0],
      value1 = value[1];
    // 日期区间格式化
    value = [value0 ? (0, _moment.default)(value0).format(formatRule) : null, value1 ? (0, _moment.default)(value1).format(formatRule) : null];
  }
  return value;
};
exports.getDateValue = getDateValue;