"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = useOptions;
var _react = require("react");
var _utils = require("../utils");
var __assign = void 0 && (void 0).__assign || function () {
  __assign = Object.assign || function (t) {
    for (var s, i = 1, n = arguments.length; i < n; i++) {
      s = arguments[i];
      for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p)) t[p] = s[p];
    }
    return t;
  };
  return __assign.apply(this, arguments);
};
var defaultTransform = {
  label: 'label',
  value: 'value',
  children: 'children',
  parentId: 'parentId',
  rootValue: null,
  keepLeaf: false
};
var defaultConfig = {
  path: [],
  params: {},
  keepOrigin: false,
  toTree: false
};
/** 递归寻找下层 */
var loop = function loop(options, level, transform, keepOrigin) {
  if (level === void 0) {
    level = 0;
  }
  return options.map(function (option, index) {
    var newOption = {
      value: undefined
    };
    if (typeof transform === 'function') {
      newOption = transform(option, index, level);
    } else {
      var _a = __assign(__assign({}, defaultTransform), transform),
        label = _a.label,
        value = _a.value,
        children = _a.children,
        parentId = _a.parentId;
      newOption = {
        label: option[label],
        value: option[value],
        children: option[children]
      };
      if (keepOrigin) {
        newOption = __assign(__assign({}, option), newOption);
      }
      if (option.hasOwnProperty(parentId)) {
        newOption[parentId] = option[parentId];
      }
    }
    // 如果还有子元素，递归下级
    if (newOption.children && newOption.children.length) {
      newOption.children = loop(newOption.children, level + 1, transform, keepOrigin);
    }
    return newOption;
  });
};
function useOptions(api, config) {
  var _a = __assign(__assign({}, defaultConfig), config),
    path = _a.path,
    params = _a.params,
    transform = _a.transform,
    _b = _a.keepOrigin,
    keepOrigin = _b === void 0 ? false : _b,
    _c = _a.toTree,
    toTree = _c === void 0 ? false : _c,
    onLoad = _a.onLoad,
    _d = _a.autoload,
    autoload = _d === void 0 ? true : _d;
  var _e = (0, _react.useState)([]),
    options = _e[0],
    setOptions = _e[1];
  var _f = (0, _react.useState)(null),
    data = _f[0],
    setData = _f[1];
  var _g = (0, _react.useState)([]),
    tree = _g[0],
    setTree = _g[1];
  var _h = (0, _react.useState)(false),
    loading = _h[0],
    setLoading = _h[1];
  var getData = function getData() {
    if (loading) {
      return;
    }
    setLoading(true);
    api(params || {}).then(function (res) {
      setData(res);
      var options = res;
      if (path && path.length) {
        path.forEach(function (key) {
          options = options[key];
        });
      }
      if (transform) {
        options = loop(options, 0, transform, keepOrigin);
      }
      setOptions(options);
      if (toTree) {
        var config_1 = __assign(__assign({}, defaultTransform), transform);
        var tree_1 = (0, _utils.listToTree)({
          data: options,
          parentKey: config_1.parentId,
          labelKey: config_1.label,
          childrenKey: config_1.children,
          valueKey: config_1.value,
          rootValue: config_1.rootValue,
          keepLeaf: config_1.keepLeaf
        });
        setTree(tree_1);
      }
      if (onLoad) {
        onLoad({
          options: options,
          tree: tree,
          data: data,
          loading: loading
        });
      }
    }).finally(function () {
      setLoading(false);
    });
  };
  (0, _react.useEffect)(function () {
    if (autoload) {
      getData();
    }
  }, []);
  return {
    options: options,
    tree: tree,
    loading: loading,
    data: data,
    load: getData
  };
}