"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.renderStatus = exports.optionObj = exports.omitObj = exports.listToTree = exports.isObj = exports.isFunc = exports.getValueByOptions = exports.getRowKey = exports.getRandomFloor = exports.getRandom = exports.getLabelByValue = exports.getKey = exports.getCurrencyValue = exports.default = exports.copy = exports.clearEmpty = void 0;
var _react = _interopRequireDefault(require("react"));
var _antd = require("antd");
var _constant = require("./constant");
function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }
var __assign = void 0 && (void 0).__assign || function () {
  __assign = Object.assign || function (t) {
    for (var s, i = 1, n = arguments.length; i < n; i++) {
      s = arguments[i];
      for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p)) t[p] = s[p];
    }
    return t;
  };
  return __assign.apply(this, arguments);
};
/**
 * 拷贝对象
 * @param obj
 */
var copy = function copy(obj) {
  return JSON.parse(JSON.stringify(obj));
};
/**
 * 获取随机数
 * @param min 最小随机数
 * @param max 最大随机数
 */
exports.copy = copy;
var getRandom = function getRandom(min, max) {
  return min + Math.random() * (max - min);
};
/**
 * 获取随机数（带小数）
 * @param min 最小随机数
 * @param max 最大随机数
 */
exports.getRandom = getRandom;
var getRandomFloor = function getRandomFloor(min, max) {
  return min + Math.floor(Math.random() * (max - min + 1));
};
/**
 * 是否是对象
 * @param obj 判断对象
 */
exports.getRandomFloor = getRandomFloor;
var isObj = function isObj(obj) {
  return Object.prototype.toString.call(obj) === '[object Object]';
};
/**
 * 是否是方法
 * @param func 判断对象
 */
exports.isObj = isObj;
var isFunc = function isFunc(func) {
  return typeof func === 'function';
};
/**
 * 清空对象的 undefined 和 null
 * @param params 要清空的对象
 */
exports.isFunc = isFunc;
var clearEmpty = function clearEmpty(params) {
  var result = {};
  for (var key in params) {
    if (params[key] != undefined) {
      result[key] = params[key];
    }
  }
  return result;
};
/**
 * 列表转树结构
 * @param props 配置参数
 */
exports.clearEmpty = clearEmpty;
var listToTree = function listToTree(props) {
  var _a = props.data,
    data = _a === void 0 ? [] : _a,
    _b = props.parentKey,
    parentKey = _b === void 0 ? 'parentId' : _b,
    _c = props.childrenKey,
    childrenKey = _c === void 0 ? 'children' : _c,
    _d = props.valueKey,
    valueKey = _d === void 0 ? 'id' : _d,
    _e = props.rootValue,
    rootValue = _e === void 0 ? null : _e,
    _f = props.keepLeaf,
    keepLeaf = _f === void 0 ? false : _f;
  var list = copy(data);
  var map = {};
  var roots = [];
  list.forEach(function (item, i) {
    map[item[valueKey]] = i;
    item[childrenKey] = [];
  });
  list.forEach(function (node) {
    var target = node[parentKey];
    if (target === rootValue) {
      roots.push(node);
    } else {
      if (map[target] !== undefined) {
        list[map[target]].children.push(node);
      } else {
        roots.push(node);
      }
    }
  });
  if (!keepLeaf) {
    list.forEach(function (item) {
      if (!item.children.length) {
        delete item.children;
      }
    });
  }
  return roots;
};
// 根据充值类型 id 获取对应的中文描述
exports.listToTree = listToTree;
var getLabelByValue = function getLabelByValue(type, options) {
  if (type === void 0) {
    type = 1;
  }
  return options.find(function (option) {
    return option.value === type;
  }).label;
};
/**
 * 数字转金额
 * @param value 金额
 */
exports.getLabelByValue = getLabelByValue;
var getCurrencyValue = function getCurrencyValue(value) {
  if (typeof value !== 'number') {
    return '0';
  }
  return value.toLocaleString();
};
/**
 * 通过选项列表把 value 变成 label
 * @param value 当前值
 * @param options 选项列表
 */
exports.getCurrencyValue = getCurrencyValue;
var getValueByOptions = function getValueByOptions(value, options) {
  var option = options.find(function (option) {
    return option.value === value;
  });
  return option ? option.label : value;
};
/**
 * 输出 YYYY-MM-DD 格式
 * @param value
 */
// export const getDate = (value: any) => {
//   return value ? moment(value).format('YYYY-MM-DD') : value
// }
/**
 * @desc渲染一个带颜色的状态
 *
 * @param status 状态值
 * @param options 选项
 *
 * @returns ReactNode
 */
exports.getValueByOptions = getValueByOptions;
var renderStatus = function renderStatus(status, options, type) {
  if (type === void 0) {
    type = 'badge';
  }
  var selectOption = options.find(function (_a) {
    var value = _a.value;
    return value === status;
  });
  if (!selectOption || !selectOption.label) {
    return status;
  }
  if (!selectOption.color && !selectOption.status) {
    return selectOption.label;
  }
  return /*#__PURE__*/_react.default.createElement("div", null, type === 'badge' ? /*#__PURE__*/_react.default.createElement(_antd.Badge, {
    color: selectOption.color,
    status: selectOption.status,
    text: selectOption.label
  }) : /*#__PURE__*/_react.default.createElement(_antd.Tag, {
    color: selectOption.color
  }, selectOption.label));
};
/**
 * 复制一个对象，只选择其中某些 key
 * @param obj 复制对象
 * @param key 要保留的 key，可以是数组
 * @returns 保留了 key 的对象
 */
exports.renderStatus = renderStatus;
var optionObj = function optionObj(obj, key) {
  var result = {};
  if (typeof key === 'string') {
    result[key] = obj[key];
  } else if (Array.isArray(key)) {
    key.forEach(function (k) {
      return result[k] = obj[k];
    });
  }
  return result;
};
/**
 * 复制一个对象，并删除某些 key
 * @param obj 复制对象
 * @param key 要删除的 key，可以是数组
 * @returns 删除了 key 的对象
 */
exports.optionObj = optionObj;
var omitObj = function omitObj(obj, key) {
  var result = __assign({}, obj);
  if (typeof key === 'string') {
    delete result[key];
  } else if (Array.isArray(key)) {
    key.forEach(function (k) {
      return delete result[k];
    });
  }
  return result;
};
/**
 * 根据行获取 rowKey，默认取 id
 * @param record 当前行
 * @param rowKey
 * @returns
 */
exports.omitObj = omitObj;
var getRowKey = function getRowKey(record, rowKey) {
  try {
    if (typeof rowKey === 'function') {
      return rowKey(record);
    } else if (typeof rowKey === 'string') {
      return rowKey;
    } else {
      return _constant.TABLE_DEFAULT_ROW_KEY;
    }
  } catch (_a) {
    return _constant.TABLE_DEFAULT_ROW_KEY;
  }
};
/**
 * 根据 rowKey 获取行的 key
 * @param record 当前行
 * @param rowKey
 * @returns
 */
exports.getRowKey = getRowKey;
var getKey = function getKey(record, rowKey) {
  return record[getRowKey(record, rowKey)];
};
exports.getKey = getKey;
var _default = {};
exports.default = _default;